
try:
    import unittest2 as ut
    assert ut  # Suppress pyflakes warning about redefinition of unused ut
except ImportError:
    import unittest as ut


from itertools import combinations
from numpy.testing import assert_array_equal, assert_array_almost_equal
import scipy as sp
from spykeutils import _scipy_quantities as spq
import quantities as pq


class TestScipyMaximum(ut.TestCase):
    def test_works_with_normal_arrays(self):
        a = sp.array([[4, 2], [2, 4]])
        b = sp.array([[3, 3], [3, 3]])
        expected = sp.array([[4, 3], [3, 4]])
        actual = spq.maximum(a, b)
        assert_array_equal(expected, actual)

    def test_works_with_quantity_arrays(self):
        a = sp.array([[4, 2], [2, 4]]) * pq.s
        b = sp.array([[3000, 3000], [3000, 3000]]) * pq.ms
        expected = sp.array([[4, 3], [3, 4]]) * pq.s
        actual = spq.maximum(a, b)
        assert_array_almost_equal(expected, actual)
        actual = spq.maximum(b, a)
        assert_array_almost_equal(expected, actual)

    def test_works_with_normal_and_quantity_arrays_mixed(self):
        a = sp.array([[4, 2], [2, 4]])
        b = sp.array([[3, 3], [3, 3]]) * pq.dimensionless
        expected = sp.array([[4, 3], [3, 4]]) * pq.dimensionless
        actual = spq.maximum(a, b)
        assert_array_almost_equal(expected, actual)
        actual = spq.maximum(b, a)
        assert_array_almost_equal(expected, actual)

    def test_works_with_(self):
        a = sp.array([[4, 2], [2, 4]]) * pq.dimensionless
        b = sp.array([[3, 3], [3, 3]]) * pq.dimensionless
        expected = sp.array([[4, 3], [3, 4]]) * pq.dimensionless
        actual = spq.maximum(a, b)
        assert_array_almost_equal(expected, actual)
        actual = spq.maximum(b, a)
        assert_array_almost_equal(expected, actual)

    def test_uses_out_param(self):
        a = sp.array([[4, 2], [2, 4]]) * pq.s
        b = sp.array([[3000, 3000], [3000, 3000]]) * pq.ms
        expected = sp.array([[4, 3], [3, 4]]) * pq.s
        out = sp.zeros_like(expected)
        assert out.units == pq.s
        spq.maximum(a, b, out)
        assert_array_almost_equal(expected, out)
        spq.maximum(b, a, out)
        assert_array_almost_equal(expected, out)


class TestScipyMinimum(ut.TestCase):
    def test_works_with_normal_arrays(self):
        a = sp.array([[4, 2], [2, 4]])
        b = sp.array([[3, 3], [3, 3]])
        expected = sp.array([[3, 2], [2, 3]])
        actual = spq.minimum(a, b)
        assert_array_equal(expected, actual)

    def test_works_with_quantity_arrays(self):
        a = sp.array([[4, 2], [2, 4]]) * pq.s
        b = sp.array([[3000, 3000], [3000, 3000]]) * pq.ms
        expected = sp.array([[3, 2], [2, 3]]) * pq.s
        actual = spq.minimum(a, b)
        assert_array_almost_equal(expected, actual)
        actual = spq.minimum(b, a)
        assert_array_almost_equal(expected, actual)

    def test_works_with_normal_and_quantity_arrays_mixed(self):
        a = sp.array([[4, 2], [2, 4]])
        b = sp.array([[3, 3], [3, 3]]) * pq.dimensionless
        expected = sp.array([[3, 2], [2, 3]]) * pq.dimensionless
        actual = spq.minimum(a, b)
        assert_array_almost_equal(expected, actual)
        actual = spq.minimum(b, a)
        assert_array_almost_equal(expected, actual)

    def test_uses_out_param(self):
        a = sp.array([[4, 2], [2, 4]]) * pq.s
        b = sp.array([[3000, 3000], [3000, 3000]]) * pq.ms
        expected = sp.array([[3, 2], [2, 3]]) * pq.s
        out = sp.zeros_like(expected)
        assert out.units == pq.s
        spq.minimum(a, b, out)
        assert_array_almost_equal(expected, out)
        spq.minimum(b, a, out)
        assert_array_almost_equal(expected, out)


class TestScipyMeshgrid(ut.TestCase):
    def test_works_with_normal_arrays(self):
        a = sp.array([1, 2, 3])
        b = sp.array([4, 5])
        expected_a = sp.array([[1, 2, 3], [1, 2, 3]])
        expected_b = sp.array([[4, 4, 4], [5, 5, 5]])
        actual_a, actual_b = spq.meshgrid(a, b)
        assert_array_equal(expected_a, actual_a)
        assert_array_equal(expected_b, actual_b)

    def test_works_with_quantity_arrays(self):
        a = sp.array([1, 2, 3]) * pq.s
        b = sp.array([4, 5]) * pq.m
        expected_a = sp.array([[1, 2, 3], [1, 2, 3]]) * pq.s
        expected_b = sp.array([[4, 4, 4], [5, 5, 5]]) * pq.m
        actual_a, actual_b = spq.meshgrid(a, b)
        assert_array_equal(expected_a, actual_a)
        assert_array_equal(expected_b, actual_b)

    def test_works_with_normal_and_quantity_arrays_mixed(self):
        a = sp.array([1, 2, 3])
        b = sp.array([4, 5]) * pq.m
        expected_a = sp.array([[1, 2, 3], [1, 2, 3]])
        expected_b = sp.array([[4, 4, 4], [5, 5, 5]]) * pq.m
        actual_a, actual_b = spq.meshgrid(a, b)
        assert_array_equal(expected_a, actual_a)
        assert_array_equal(expected_b, actual_b)

        a = sp.array([1, 2, 3]) * pq.s
        b = sp.array([4, 5])
        expected_a = sp.array([[1, 2, 3], [1, 2, 3]]) * pq.s
        expected_b = sp.array([[4, 4, 4], [5, 5, 5]])
        actual_a, actual_b = spq.meshgrid(a, b)
        assert_array_equal(expected_a, actual_a)
        assert_array_equal(expected_b, actual_b)


class TestScipyConcatenate(ut.TestCase):
    def test_works_with_normal_arrays(self):
        a = sp.array([[1]])
        b = sp.array([[2]])
        c = sp.array([[3]])
        axis = 1
        expected = sp.array([[1, 2, 3]])
        actual = spq.concatenate((a, b, c), axis=axis)
        assert_array_equal(expected, actual)

    def test_works_with_quantity_arrays(self):
        a = sp.array([[1]]) * pq.s
        b = sp.array([[2000]]) * pq.ms
        c = sp.array([[3]]) * pq.s
        axis = 1
        expected = sp.array([[1, 2, 3]]) * pq.s
        actual = spq.concatenate((a, b, c), axis=axis)
        assert_array_equal(expected, actual)

    def test_works_with_normal_and_quantity_arrays_mixed(self):
        a = sp.array([[1]])
        b = sp.array([[2]]) * pq.dimensionless
        c = sp.array([[3]])
        axis = 1
        expected = sp.array([[1, 2, 3]])
        actual = spq.concatenate((a, b, c), axis=axis)
        assert_array_equal(expected, actual)

    def test_raises_exception_if_mixing_incompatible_units(self):
        a = sp.array([[1]])
        b = sp.array([[2]]) * pq.dimensionless
        c = sp.array([[3]]) * pq.s
        d = sp.array([[4]]) * pq.m
        for p in combinations((a, c, d), 2):
            with self.assertRaises(Exception):
                spq.concatenate(p)
            with self.assertRaises(Exception):
                spq.concatenate(p[::-1])
        for p in combinations((b, c, d), 2):
            with self.assertRaises(Exception):
                spq.concatenate(p)
            with self.assertRaises(Exception):
                spq.concatenate(p[::-1])


class TestScipyInner(ut.TestCase):
    def test_works_with_normal_arrays(self):
        a = sp.array([0.0, 1.0, 2.0])
        b = sp.array([2.0, 2.0, 3.0])
        expected = sp.inner(a, b)
        actual = spq.inner(a, b)
        self.assertAlmostEqual(expected, actual)

    def test_works_with_quantities_arrays(self):
        a = sp.array([0.0, 1.0, 2.0]) * pq.s
        b = sp.array([2000.0, 2000.0, 3000.0]) * pq.ms
        expected = 8.0 * pq.s
        actual = spq.inner(a, b)
        self.assertAlmostEqual(expected, actual.rescale(expected.units))

    def test_works_with_normal_and_quantity_arrays_mixed(self):
        a = sp.array([0.0, 1.0, 2.0]) * pq.dimensionless
        b = sp.array([2.0, 2.0, 3.0])
        expected = sp.inner(a, b)
        actual = spq.inner(a, b)
        self.assertAlmostEqual(expected, actual)

    def test_raises_exception_if_mixing_incompatible_units(self):
        a = sp.array([1.0])
        b = sp.array([2.0]) * pq.dimensionless
        c = sp.array([3.0]) * pq.s
        d = sp.array([4.0]) * pq.m
        for p in combinations((a, c, d), 2):
            with self.assertRaises(Exception):
                spq.inner(p)
            with self.assertRaises(Exception):
                spq.inner(p[::-1])
        for p in combinations((b, c, d), 2):
            with self.assertRaises(Exception):
                spq.inner(p)
            with self.assertRaises(Exception):
                spq.inner(p[::-1])

    def test_works_with_multidimensional_arrays(self):
        a = sp.array([[0.0, 1.0], [2.0, 3.0]]) * pq.s
        b = sp.array([[2.0, 2.0], [3.0, 4.0]]) * pq.s
        expected = sp.array([[2.0, 4.0], [10.0, 18.0]]) * pq.s
        actual = spq.inner(a, b)
        assert_array_almost_equal(expected, actual.rescale(expected.units))


class TestScipyDiag(ut.TestCase):
    def test_diag_of_unitless_1d_array(self):
        a = sp.array([1, 2])
        expected = sp.array([[1, 0], [0, 2]])
        actual = spq.diag(a)
        assert_array_equal(expected, actual)

    def test_diag_of_unitless_2d_array(self):
        a = sp.array([[1, 0], [0, 2]])
        expected = sp.array([1, 2])
        actual = spq.diag(a)
        assert_array_equal(expected, actual)

    def test_diag_of_1d_quantity(self):
        a = sp.array([1, 2]) * pq.s
        expected = sp.array([[1, 0], [0, 2]]) * pq.s
        actual = spq.diag(a)
        self.assertEqual(expected.units, actual.units)
        assert_array_equal(expected, actual)

    def test_diag_of_2d_quantity(self):
        a = sp.array([[1, 0], [0, 2]]) * pq.s
        expected = sp.array([1, 2]) * pq.s
        actual = spq.diag(a)
        self.assertEqual(expected.units, actual.units)
        assert_array_equal(expected, actual)

    def test_respects_k_argument(self):
        a = sp.array([[1, 5], [0, 2]]) * pq.s
        expected = sp.array([5]) * pq.s
        actual = spq.diag(a, k=1)
        self.assertEqual(expected.units, actual.units)
        assert_array_equal(expected, actual)


class Test_linspace(ut.TestCase):
    def test_returns_correct_quantity_for_one_bin(self):
        expected = sp.array([1.0]) * pq.s
        actual = spq.linspace(1.0 * pq.s, 4.0 * pq.s, 1)
        assert_array_almost_equal(expected, actual)

    def test_returns_correct_quantity_for_one_bin_after_rounding(self):
        expected = sp.array([1.0]) * pq.s
        actual = spq.linspace(1.0 * pq.s, 4.0 * pq.s, 1.4)
        assert_array_almost_equal(expected, actual)

    def test_returns_correct_quantity_for_more_than_one_bin(self):
        expected = sp.array([1.0, 2.0, 3.0, 4.0]) * pq.s
        actual = spq.linspace(1.0 * pq.s, 4.0 * pq.s, 4)
        assert_array_almost_equal(expected, actual)


if __name__ == '__main__':
    ut.main()



import functools

class CancelException(Exception):
    """ This is raised when a user cancels a progress process. It is used
    by :class:`ProgressIndicator` and its descendants.
    """
    pass


def ignores_cancel(function):
    """ Decorator for functions that should ignore a raised
    :class:`CancelException` and just return nothing in this case
    """
    @functools.wraps(function)
    def inner(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except CancelException:
            return
    return inner


class ProgressIndicator(object):
    """ Base class for classes indicating progress of a long operation.

    This class does not implement any of the methods and can be used
    as a dummy if no progress indication is needed.
    """

    def set_ticks(self, ticks):
        """ Set the required number of ticks before the operation is done.

        :param int ticks: The number of steps that the operation will take.
        """
        pass

    def begin(self, title=''):
        """ Signal that the operation starts.

        :param string title: The name of the whole operation.
        """
        pass

    def step(self, num_steps=1):
        """ Signal that one or more steps of the operation were completed.

        :param int num_steps: The number of steps that have been completed.
        """
        pass

    def set_status(self, new_status):
        """ Set status description.

        :param string new_status: A description of the current status.
        """
        pass

    def done(self):
        """ Signal that the operation is done. """
        pass


import scipy as sp
from collections import OrderedDict

import quantities as pq

from progress_indicator import ProgressIndicator
from . import SpykeException


def correlogram(trains, bin_size, max_lag=500 * pq.ms, border_correction=True,
                per_second=True, unit=pq.ms, progress=None):
    """ Return (cross-)correlograms from a dictionary of spike train
    lists for different units.

    :param dict trains: Dictionary of :class:`neo.core.SpikeTrain` lists.
    :param bin_size: Bin size (time).
    :type bin_size: Quantity scalar
    :param max_lag: Cut off (end time of calculated correlogram).
    :type max_lag: Quantity scalar
    :param bool border_correction: Apply correction for less data at higher
        timelags. Not perfect for bin_size != 1*``unit``, especially with
        large ``max_lag`` compared to length of spike trains.
    :param bool per_second: If ``True``, counts returned are per second.
        Otherwise, counts per spike train are returned.
    :param Quantity unit: Unit of X-Axis.
    :param progress: A ProgressIndicator object for the operation.
    :type progress: :class:`.progress_indicator.ProgressIndicator`
    :returns: Two values:

        * An ordered dictionary indexed with the indices of ``trains`` of
          ordered dictionaries indexed with the same indices. Entries of
          the inner dictionaries are the resulting (cross-)correlograms as
          numpy arrays. All crosscorrelograms can be indexed in two
          different ways: ``c[index1][index2]`` and ``c[index2][index1]``.
        * The bins used for the correlogram calculation.
    :rtype: dict, Quantity 1D
    """
    if not progress:
        progress = ProgressIndicator()

    bin_size.rescale(unit)
    max_lag.rescale(unit)

    # Create bins, making sure that 0 is at the center of central bin
    half_bins = sp.arange(bin_size / 2, max_lag, bin_size)
    all_bins = list(reversed(-half_bins))
    all_bins.extend(half_bins)
    bins = sp.array(all_bins) * unit
    middle_bin = len(bins) / 2 - 1

    indices = trains.keys()
    num_trains = len(trains[indices[0]])
    if not num_trains:
        raise SpykeException('Could not create correlogram: No spike trains!')
    for u in range(1, len(indices)):
        if len(trains[indices[u]]) != num_trains:
            raise SpykeException('Could not create correlogram: All units ' +
                                 'need the same number of spike trains!')

    progress.set_ticks(sp.sum(range(len(trains) + 1) * num_trains))

    corrector = 1
    if border_correction:
        # Need safe min/max functions
        def safe_max(seq):
            if len(seq) < 1:
                return 0
            return max(seq)

        def safe_min(seq):
            if len(seq) < 1:
                return 2 ** 22  # Some arbitrary large value
            return min(seq)

        max_w = max([max([safe_max(t) for t in l])
                     for l in trains.itervalues()])
        min_w = min([min([safe_min(t) for t in l])
                     for l in trains.itervalues()])

        train_length = (max_w - min_w)
        l = int(round(middle_bin)) + 1
        cE = max(train_length - (l * bin_size) + 1 * unit, 1 * unit)

        corrector = (train_length / sp.concatenate(
            (sp.linspace(cE, train_length, l - 1, False),
             sp.linspace(train_length, cE, l)))).magnitude

    correlograms = OrderedDict()
    for i1 in xrange(len(indices)):  # For each index
        # For all later indices, including itself
        for i2 in xrange(i1, len(indices)):
            histogram = sp.zeros(len(bins) - 1)
            for t in xrange(num_trains):
                train1 = trains[indices[i1]][t].rescale(unit).reshape((1, -1))
                train2 = trains[indices[i2]][t].rescale(unit).reshape((-1, 1))
                histogram += sp.histogram(
                    sp.subtract(train1, train2), bins=bins)[0]
                if i1 == i2:  # Correction for autocorrelogram
                    histogram[middle_bin] -= len(train2)
                progress.step()

            if per_second:
                l = train1.t_stop - train1.t_start
                if train2.t_stop - train2.t_start != l:
                    raise SpykeException(
                        'A spike train pair does not have equal length,'
                        'cannot calculate count per second.')
                histogram /= l.rescale(pq.s)

            crg = corrector * histogram / num_trains
            if indices[i1] not in correlograms:
                correlograms[indices[i1]] = OrderedDict()
            correlograms[indices[i1]][indices[i2]] = crg
            if i1 != i2:
                if indices[i2] not in correlograms:
                    correlograms[indices[i2]] = OrderedDict()
                correlograms[indices[i2]][indices[i1]] = crg[::-1]

    return correlograms, bins


import neo
import numpy.random
import quantities as pq
import scipy as sp
import _scipy_quantities as spq


def gen_homogeneous_poisson(
        rate, t_start=0 * pq.s, t_stop=None, max_spikes=None,
        refractory=0 * pq.s):
    """ Generate a homogeneous Poisson spike train. The length is controlled
    with `t_stop` and `max_spikes`. Either one or both of these arguments have
    to be given.

    :param rate: Average firing rate of the spike train to generate as
        frequency scalar.
    :type rate: Quantity scalar
    :param t_start: Time at which the spike train begins as time scalar. The
        first actual spike will be greater than this time.
    :type t_start: Quantity scalar
    :param t_stop: Time at which the spike train ends as time scalar. All
        generated spikes will be lower or equal than this time. If set to None,
        the number of generated spikes is controlled by `max_spikes` and
        `t_stop` will be equal to the last generated spike.
    :type t_stop: Quantity scalar
    :param max_spikes: Maximum number of spikes to generate. Fewer spikes might
        be generated in case `t_stop` is also set.
    :param refractory: Absolute refractory period as time scalar. No spike will
        follow another spike for the given duration. Afterwards the firing rate
        will instantaneously be set to `rate` again.
    :type refractory: Quantity scalar

    :returns: The generated spike train.
    :rtype: :class:`neo.core.SpikeTrain`
    """

    if t_stop is None and max_spikes is None:
        raise ValueError('Either t_stop or max_spikes has to be set.')

    if max_spikes is not None:
        spike_times = sp.cumsum(numpy.random.exponential(
            rate ** -1, max_spikes)) * (rate.units ** -1).simplified
        spike_times += t_start
        if refractory > 0:
            spike_times += sp.arange(spike_times.size) * refractory
        if t_stop is not None:
            spike_times = spike_times[spike_times <= t_stop]
    else:
        scale = (rate ** -1).rescale(t_stop.units)
        trains = []
        last_spike = t_start.rescale(t_stop.units)
        while last_spike < t_stop:
            # Generate a bit more than the average number of expected spike to
            # be finished in most cases in one loop. The factor was determined
            # empirically.
            num_spikes = int(1.7 * (
                (t_stop - last_spike) * rate).simplified) + 1
            train = sp.cumsum(numpy.random.exponential(scale, num_spikes)) * \
                scale.units + last_spike
            if refractory > 0:
                train += sp.arange(train.size) * refractory
            if train.size > 0:
                last_spike = train[-1]
                if last_spike >= t_stop:
                    train = train[train < t_stop]
                trains.append(train)
        spike_times = spq.concatenate(trains)

    if t_stop is None:
        t_stop = spike_times[-1]
    return neo.SpikeTrain(spike_times, t_start=t_start, t_stop=t_stop)


def gen_inhomogeneous_poisson(
        modulation, max_rate, t_start=0 * pq.s, t_stop=None, max_spikes=None,
        refractory=0 * pq.s):
    """ Generate an inhomogeneous Poisson spike train. The length is controlled
    with `t_stop` and `max_spikes`. Either one or both of these arguments have
    to be given.

    :param function modulation: Function :math:`f((t_1, \\dots, t_n)):
        [\\text{t\\_start}, \\text{t\\_end}]^n \\rightarrow [0, 1]^n` giving
        the instantaneous firing rates at times :math:`(t_1, \\dots, t_n)` as
        proportion of `max_rate`. Thus, a 1-D array will be passed to the
        function and it should return an array of the same size.
    :param max_rate: Maximum firing rate of the spike train to generate as
        frequency scalar.
    :type max_rate: Quantity scalar
    :param t_start: Time at which the spike train begins as time scalar. The
        first actual spike will be greater than this time.
    :type t_start: Quantity scalar
    :param t_stop: Time at which the spike train ends as time scalar. All
        generated spikes will be lower or equal than this time. If set to None,
        the number of generated spikes is controlled by `max_spikes` and
        `t_stop` will be equal to the last generated spike.
    :type t_stop: Quantity scalar
    :param refractory: Absolute refractory period as time scalar. No spike will
        follow another spike for the given duration. Afterwards the firing rate
        will instantaneously be set to `rate` again.
    :type refractory: Quantity scalar

    :returns: The generated spike train.
    :rtype: :class:`neo.core.SpikeTrain`
    """

    st = gen_homogeneous_poisson(
        max_rate, t_start, t_stop, max_spikes, refractory)
    return st[numpy.random.rand(st.size) < modulation(st)]



from __future__ import division

import scipy as sp
import quantities as pq
import neo
from progress_indicator import ProgressIndicator
import signal_processing as sigproc
import tools
import copy as cp
from . import SpykeException


def psth(
        trains, bin_size, rate_correction=True, start=0 * pq.ms,
        stop=sp.inf * pq.s):
    """ Return dictionary of peri stimulus time histograms for a dictionary
    of spike train lists.

    :param dict trains: A dictionary of lists of :class:`neo.core.SpikeTrain`
        objects.
    :param bin_size: The desired bin size (as a time quantity).
    :type bin_size: Quantity scalar
    :param bool rate_correction: Determines if a rates (``True``) or
        counts (``False``) are returned.
    :param start: The desired time for the start of the first bin. It
        will be recalculated if there are spike trains which start
        later than this time.
    :type start: Quantity scalar
    :param stop: The desired time for the end of the last bin. It will
        be recalculated if there are spike trains which end earlier
        than this time.
    :type stop: Quantity scalar
    :returns: A dictionary (with the same indices as ``trains``) of arrays
        containing counts (or rates if ``rate_correction`` is ``True``)
        and the bin borders.
    :rtype: dict, Quantity 1D
    """
    if not trains:
        raise SpykeException('No spike trains for PSTH!')

    start, stop = tools.minimum_spike_train_interval(trains, start, stop)
    binned, bins = tools.bin_spike_trains(trains, 1.0 / bin_size, start, stop)

    cumulative = {}
    time_multiplier = 1.0 / float(bin_size.rescale(pq.s))
    for u in binned:
        if rate_correction:
            cumulative[u] = sp.mean(sp.array(binned[u]), 0)
        else:
            cumulative[u] = sp.sum(sp.array(binned[u]), 0)
        cumulative[u] *= time_multiplier

    return cumulative, bins


def aligned_spike_trains(trains, events, copy=True):
    """ Return a list of spike trains aligned to an event (the event will
    be time 0 on the returned trains).

    :param list trains: A list of :class:`neo.core.SpikeTrain` objects.
    :param dict events: A dictionary of Event objects, indexed by segment.
        These events will be used to align the spike trains and will be at 
        time 0 for the aligned spike trains.
    :param bool copy: Determines if aligned copies of the original
        spike trains  will be returned. If not, every spike train needs
        exactly one corresponding event, otherwise a ``ValueError`` will
        be raised. Otherwise, entries with no event will be ignored.
    """
    ret = []
    for t in trains:
        s = t.segment
        if s not in events:
            if not copy:
                raise ValueError(
                    'Cannot align spike trains: At least one segment does' +
                    'not have an align event.')
            continue

        e = events[s]

        if copy:
            st = neo.SpikeTrain(
                t, t.t_stop, units=t.units,
                sampling_rate=t.sampling_rate, t_start=t.t_start,
                waveforms=t.waveforms, left_sweep=t.left_sweep,
                name=t.name, file_origin=t.file_origin,
                description=t.description, **t.annotations)
        else:
            st = t

        st -= e.time
        st.t_stop -= e.time
        st.t_start -= e.time
        ret.append(st)

    return ret


def spike_density_estimation(trains, start=0 * pq.ms, stop=None,
                             kernel=None, kernel_size=100 * pq.ms,
                             optimize_steps=None, progress=None):
    """ Create a spike density estimation from a dictionary of
    lists of spike trains.

    The spike density estimations give an estimate of the instantaneous
    rate. The density estimation is evaluated at 1024 equally spaced
    points covering the range of the input spike trains. Optionally finds
    optimal kernel size for given data using the algorithm from
    (Shimazaki, Shinomoto. Journal of Computational Neuroscience. 2010).

    :param dict trains: A dictionary of :class:`neo.core.SpikeTrain` lists.
    :param start: The desired time for the start of the estimation. It
        will be recalculated if there are spike trains which start later
        than this time. This parameter can be negative (which could be
        useful when aligning on events).
    :type start: Quantity scalar
    :param stop: The desired time for the end of the estimation. It will
        be recalculated if there are spike trains which end earlier
        than this time.
    :type stop: Quantity scalar
    :param kernel: The kernel function or instance to use, should accept
        two parameters: A ndarray of distances and a kernel size.
        The total area under the kernel function should be 1.
        Automatic optimization assumes a Gaussian kernel and will
        likely not produce optimal results for different kernels.
        Default: Gaussian kernel
    :type kernel: func or :class:`.signal_processing.Kernel`
    :param kernel_size: A uniform kernel size for all spike trains.
            Only used if optimization of kernel sizes is not used.
    :type kernel_size: Quantity scalar
    :param optimize_steps: An array of time lengths that will be
        considered in the kernel width optimization. Note that the
        optimization assumes a Gaussian kernel and will most likely
        not give the optimal kernel size if another kernel is used.
        If None, ``kernel_size`` will be used.
    :type optimize_steps: Quantity 1D
    :param progress: Set this parameter to report progress.
    :type progress: :class:`.progress_indicator.ProgressIndicator`

    :returns: Three values:

        * A dictionary of the spike density estimations (Quantity 1D in
          Hz). Indexed the same as ``trains``.
        * A dictionary of kernel sizes (Quantity scalars). Indexed the
          same as ``trains``.
        * The used evaluation points.
    :rtype: dict, dict, Quantity 1D
    """
    if not progress:
        progress = ProgressIndicator()

    if optimize_steps is None or len(optimize_steps) < 1:
        units = kernel_size.units
    else:
        units = optimize_steps.units

    if kernel is None:
        kernel = sigproc.GaussianKernel(100 * pq.ms)

    # Prepare evaluation points
    max_start, max_stop = tools.minimum_spike_train_interval(trains)

    start = max(start, max_start)
    start.units = units
    if stop is not None:
        stop = min(stop, max_stop)
    else:
        stop = max_stop
    stop.units = units
    bins = sp.linspace(start, stop, 1025)
    eval_points = bins[:-1] + (bins[1] - bins[0]) / 2

    if optimize_steps is None or len(optimize_steps) < 1:
        kernel_size = {u: kernel_size for u in trains}
    else:
        # Find optimal kernel size for all spike train sets
        progress.set_ticks(len(optimize_steps) * len(trains))
        progress.set_status('Calculating optimal kernel size')
        kernel_size = {}
        for u, t in trains.iteritems():
            c = collapsed_spike_trains(t)
            kernel_size[u] = optimal_gauss_kernel_size(
                c.time_slice(start, stop), optimize_steps, progress)

    progress.set_ticks(len(trains))
    progress.set_status('Creating spike density plot')

    # Calculate KDEs
    kde = {}
    for u, t in trains.iteritems():
        # Collapse spike trains
        collapsed = collapsed_spike_trains(t).rescale(units)
        scaled_kernel = sigproc.as_kernel_of_size(kernel, kernel_size[u])

        # Create density estimation using convolution
        sliced = collapsed.time_slice(start, stop)
        sampling_rate = 1024.0 / (sliced.t_stop - sliced.t_start)
        kde[u] = sigproc.st_convolve(
            sliced, scaled_kernel, sampling_rate,
            kernel_discretization_params={
                'num_bins': 2048, 'ensure_unit_area': True})[0] / len(trains[u])
        kde[u].units = pq.Hz
    return kde, kernel_size, eval_points


def collapsed_spike_trains(trains):
    """ Return a superposition of a list of spike trains.

    :param iterable trains: A list of :class:`neo.core.SpikeTrain` objects
    :returns: A spike train object containing all spikes of the given
        spike trains.
    :rtype: :class:`neo.core.SpikeTrain`
    """
    if not trains:
        return neo.SpikeTrain([], 0 * pq.s)

    start = min((t.t_start for t in trains))
    stop = max((t.t_stop for t in trains))

    collapsed = []
    for t in trains:
        collapsed.extend(sp.asarray(t.rescale(stop.units)))

    return neo.SpikeTrain(collapsed * stop.units, t_stop=stop, t_start=start)


def optimal_gauss_kernel_size(train, optimize_steps, progress=None):
    """ Return the optimal kernel size for a spike density estimation
    of a spike train for a gaussian kernel. This function takes a single
    spike train, which can be a superposition of multiple spike trains
    (created with :func:`collapsed_spike_trains`) that should be included
    in a spike density estimation.

    Implements the algorithm from
    (Shimazaki, Shinomoto. Journal of Computational Neuroscience. 2010).

    :param train: The spike train for which the kernel
        size should be optimized.
    :type train: :class:`neo.core.SpikeTrain`
    :param optimize_steps: Array of kernel sizes to try (the best of
        these sizes will be returned).
    :type optimize_steps: Quantity 1D
    :param progress: Set this parameter to report progress. Will be
        advanced by len(`optimize_steps`) steps.
    :type progress: :class:`.progress_indicator.ProgressIndicator`
    :returns: Best of the given kernel sizes
    :rtype: Quantity scalar
    """
    if not progress:
        progress = ProgressIndicator()

    x = train.rescale(optimize_steps.units)

    N = len(train)
    C = {}

    sampling_rate = 1024.0 / (x.t_stop - x.t_start)
    dt = float(1.0 / sampling_rate)
    y_hist = tools.bin_spike_trains({0: [x]}, sampling_rate)[0][0][0]
    y_hist = sp.asfarray(y_hist) / N / dt
    for step in optimize_steps:
        s = float(step)
        yh = sigproc.smooth(
            y_hist, sigproc.GaussianKernel(2 * step), sampling_rate, num_bins=2048,
            ensure_unit_area=True) * optimize_steps.units

        # Equation from Matlab code, 7/2012
        c = (sp.sum(yh ** 2) * dt -
             2 * sp.sum(yh * y_hist) * dt +
             2 * 1 / sp.sqrt(2 * sp.pi) / s / N)
        C[s] = c * N * N
        progress.step()

    # Return kernel size with smallest cost
    return min(C, key=C.get) * optimize_steps.units


import scipy as sp
import quantities as pq

from guiqwt.builder import make
from guiqwt.baseplot import BasePlot
from guiqwt.plot import BaseCurveWidget
from PyQt4 import Qt

from ..progress_indicator import ProgressIndicator
from .. import SpykeException
from dialog import PlotDialog
import helper


@helper.needs_qt
def spikes(spikes, axes_style, strong=None, anti_alias=False, fade=1.0,
           subplot_layout=0, time_unit=pq.ms, progress=None):
    """ Create a plot dialog with spike waveforms. Assumes that all spikes
    have waveforms with the same number of channels.

    :param dict spikes: A dictionary of :class:`neo.core.Spike` lists.
    :param int axes_style: Plotting mode. The following values are possible:

        - 1: Show each channel in a seperate plot, split vertically.
        - 2: Show each channel in a separate plot, split horizontally.
        - 3: Show each key of ``spikes`` in a separate plot,
          channels are split vertically.
        - 4: Show each key of ``spikes`` in a separate plot,
          channels are split horizontally.
        - 5: Show all channels in the same plot, split vertically.
        - 6: Show all channels in the same plot, split horizontally.

    :param dict strong: A dictionary of :class:`neo.core.Spike` lists. When
        given, these spikes are shown as thick lines on top of the regular
        spikes in the respective plots.
    :param bool anti_alias: Determines whether an antialiased plot is created.
    :param float fade: Vary transparency by segment. For values > 0, the first
        spike for each unit is displayed with the corresponding alpha
        value and alpha is linearly interpolated until it is 1 for the
        last spike. For values < 0, alpha is 1 for the first spike and
        ``fade`` for the last spike. Does not affect spikes from ``strong``.
    :param bool subplot_layout: The way subplots are arranged on the window:

        - 0: Linear - horizontally or vertically,
          depending on ``axis_style``.
        - 1: Square - this layout tries to have the same number of plots per
          row and per column.

    :param Quantity time_unit: Unit of X-Axis.
    :param progress: Set this parameter to report progress.
    :type progress: :class:`spykeutils.progress_indicator.ProgressIndicator`
    """
    if (not spikes or sum((len(l) for l in spikes.itervalues())) < 1) and \
            (not strong or sum((len(l) for l in strong.itervalues())) < 1):
        raise SpykeException('No spikes for spike waveform plot!')
    if not progress:
        progress = ProgressIndicator()
    if strong is None:
        strong = {}

    progress.begin('Creating waveform plot')
    progress.set_ticks(
        sum((len(l) for l in spikes.itervalues())) +
        sum((len(l) for l in strong.itervalues())))
    win_title = 'Spike waveforms'
    win = PlotDialog(toolbar=True, wintitle=win_title)

    try:
        ref_spike = spikes[spikes.keys()[0]][0]
    except IndexError:
        ref_spike = strong[strong.keys()[0]][0]
    if ref_spike.waveform is None:
        raise SpykeException(
            'Cannot create waveform plot: At least one spike '
            'has no waveform or sampling rate!')
    ref_units = ref_spike.waveform.units
    channels = range(ref_spike.waveform.shape[1])

    # Keys from spikes and strong without duplicates in original order
    seen = set()
    indices = [k for k in spikes.keys() + strong.keys()
               if k not in seen and not seen.add(k)]

    if axes_style <= 2:  # Separate channel plots
        for c in channels:
            pw = BaseCurveWidget(win)
            plot = pw.plot
            plot.set_antialiasing(anti_alias)
            for u in spikes:
                color = helper.get_object_color(u)
                qcol = Qt.QColor(color)
                alpha = fade if fade > 0.0 else 1.0
                alpha_step = 1.0 - fade if fade > 0.0 else -1.0 - fade
                alpha_step /= len(spikes[u])
                if len(spikes[u]) == 1:
                    alpha = 1.0

                for s in spikes[u]:
                    if s.waveform is None or s.sampling_rate is None:
                        raise SpykeException(
                            'Cannot create waveform plot: '
                            'At least one spike has no '
                            'waveform or sampling rate!')
                    x = (sp.arange(s.waveform.shape[0]) /
                         s.sampling_rate).rescale(time_unit)
                    curve = make.curve(
                        x, s.waveform[:, c].rescale(ref_units),
                        title=u.name, color=color)

                    qcol.setAlphaF(alpha)
                    curve.setPen(Qt.QPen(qcol))
                    alpha += alpha_step

                    plot.add_item(curve)
                    progress.step()

            for u in strong:
                color = helper.get_object_color(u)
                for s in strong[u]:
                    x = (sp.arange(s.waveform.shape[0]) /
                         s.sampling_rate).rescale(time_unit)
                    outline = make.curve(
                        x, s.waveform[:, c].rescale(ref_units),
                        color='#000000', linewidth=4)
                    curve = make.curve(
                        x, s.waveform[:, c].rescale(ref_units),
                        color=color, linewidth=2)
                    plot.add_item(outline)
                    plot.add_item(curve)
                    progress.step()

            _add_plot(plot, pw, win, c, len(channels), subplot_layout,
                      axes_style, time_unit, ref_units)

        helper.make_window_legend(win, indices, True)
    elif axes_style > 4:  # Only one plot needed
        pw = BaseCurveWidget(win)
        plot = pw.plot
        plot.set_antialiasing(anti_alias)

        if axes_style == 6:  # Horizontal split
            l = _split_plot_hor(channels, spikes, strong, fade, ref_units,
                                time_unit, progress, plot)

            plot.set_axis_title(BasePlot.X_BOTTOM, 'Time')
            plot.set_axis_unit(
                BasePlot.X_BOTTOM, time_unit.dimensionality.string)
        else:  # Vertical split
            channels.reverse()

            max_offset = _find_y_offset(channels, spikes, strong, ref_units)
            l = _split_plot_ver(channels, spikes, strong, fade, ref_units,
                                time_unit, progress, max_offset, plot)

            plot.set_axis_title(BasePlot.Y_LEFT, 'Voltage')
            plot.set_axis_unit(
                BasePlot.Y_LEFT, ref_units.dimensionality.string)

        win.add_plot_widget(pw, 0)
        win.add_legend_option([l], True)
    else:  # One plot per unit
        if axes_style == 3:
            channels.reverse()
            max_offset = _find_y_offset(channels, spikes, strong, ref_units)

        for i, u in enumerate(indices):
            pw = BaseCurveWidget(win)
            plot = pw.plot
            plot.set_antialiasing(anti_alias)

            spk = {}
            if u in spikes:
                spk[u] = spikes[u]
            st = {}
            if u in strong:
                st[u] = strong[u]

            if axes_style == 3:  # Vertical split
                _split_plot_ver(channels, spk, st, fade, ref_units,
                                time_unit, progress, max_offset, plot)
            else:  # Horizontal split
                _split_plot_hor(channels, spk, st, fade, ref_units,
                                time_unit, progress, plot)

            _add_plot(plot, pw, win, i, len(indices), subplot_layout,
                      axes_style, time_unit, ref_units)

    win.add_custom_curve_tools()
    progress.done()
    win.show()

    if axes_style <= 2:
        if len(channels) > 1:
            win.add_x_synchronization_option(True, channels)
            win.add_y_synchronization_option(True, channels)
    elif axes_style <= 4:
        if len(spikes) > 1:
            win.add_x_synchronization_option(True, range(len(spikes)))
            win.add_y_synchronization_option(True, range(len(spikes)))

    return win


def _add_plot(plot, widget, win, index, total, subplot_layout, axes_style,
              time_unit, ref_units):
    """ Add a plot to the window in the right position with the correct axis
    labels.
    """
    if subplot_layout == 0:
        if axes_style == 1 or axes_style == 4 or index == 0:
            plot.set_axis_title(BasePlot.Y_LEFT, 'Voltage')
            plot.set_axis_unit(
                BasePlot.Y_LEFT, ref_units.dimensionality.string)
        if axes_style == 2 or axes_style == 3 or index == total - 1:
            plot.set_axis_title(BasePlot.X_BOTTOM, 'Time')
            plot.set_axis_unit(
                BasePlot.X_BOTTOM, time_unit.dimensionality.string)
        if axes_style == 1 or axes_style == 4:
            win.add_plot_widget(widget, index)
        else:
            win.add_plot_widget(widget, index, 0, index)
    else:
        size = int(sp.sqrt(total) + 0.99)
        if index % size == 0:
            plot.set_axis_title(BasePlot.Y_LEFT, 'Voltage')
            plot.set_axis_unit(
                BasePlot.Y_LEFT, ref_units.dimensionality.string)
        if index >= total - size:
            plot.set_axis_title(BasePlot.X_BOTTOM, 'Time')
            plot.set_axis_unit(
                BasePlot.X_BOTTOM, time_unit.dimensionality.string)
        win.add_plot_widget(widget, index, index / size, index % size)


def _find_y_offset(channels, spikes, strong, ref_units):
    """ Find y offset needed when plotting spikes split vertically by channel.
    """
    max_y = []
    min_y = []
    if spikes:
        for i, c in enumerate(channels):
            max_y.append(
                max(max(s.waveform[:, c].max() for s in d)
                    for d in spikes.itervalues()))
            min_y.append(
                min(min(s.waveform[:, c].min() for s in d)
                    for d in spikes.itervalues()))

    if strong:
        for i, c in enumerate(channels):
            max_y.append(
                max(max(s.waveform[:, c].max() for s in d)
                    for d in strong.itervalues()))
            min_y.append(
                min(min(s.waveform[:, c].min() for s in d)
                    for d in strong.itervalues()))

    max_offset = 0 * ref_units
    for i in range(1, len(channels)):
        offset = max_y[i - 1] - min_y[i]
        if offset > max_offset:
            max_offset = offset

    return max_offset


def _split_plot_ver(channels, spikes, strong, fade, ref_units, time_unit,
                    progress, max_offset, plot):
    """ Fill a plot with spikes vertically split by channel. Returns legend.
    """
    offset = 0 * ref_units

    for c in channels:
        for u in spikes:
            color = helper.get_object_color(u)
            qcol = Qt.QColor(color)
            alpha = fade if fade > 0.0 else 1.0
            alpha_step = 1.0 - fade if fade > 0.0 else -1.0 - fade
            alpha_step /= len(spikes[u])
            if len(spikes[u]) == 1:
                alpha = 1.0

            for s in spikes[u]:
                if s.waveform is None or s.sampling_rate is None:
                    raise SpykeException('Cannot create waveform plot: '
                                         'At least one spike has no '
                                         'waveform or sampling rate!')
                x = (sp.arange(s.waveform.shape[0]) /
                     s.sampling_rate).rescale(time_unit)
                curve = make.curve(
                    x,
                    s.waveform[:, c].rescale(ref_units) + offset,
                    u.name, color=color)

                qcol.setAlphaF(alpha)
                curve.setPen(Qt.QPen(qcol))
                alpha += alpha_step

                plot.add_item(curve)
                progress.step()

        for u in strong:
            color = helper.get_object_color(u)
            for s in strong[u]:
                x = (sp.arange(s.waveform.shape[0]) /
                     s.sampling_rate).rescale(time_unit)
                outline = make.curve(
                    x, s.waveform[:, c].rescale(ref_units) + offset,
                    color='#000000', linewidth=4)
                curve = make.curve(
                    x, s.waveform[:, c].rescale(ref_units) + offset,
                    color=color, linewidth=2)
                plot.add_item(outline)
                plot.add_item(curve)
                progress.step()

        offset += max_offset

    l = _add_legend(plot, spikes, strong)

    return l


def _split_plot_hor(channels, spikes, strong, fade, ref_units, time_unit,
                    progress, plot):
    """ Fill a plot with spikeshorizontally split by channel. Returns legend.
    """
    offset = 0 * time_unit

    for c in channels:
        x_off = 0 * time_unit
        for u in spikes:
            color = helper.get_object_color(u)
            qcol = Qt.QColor(color)
            alpha = fade if fade > 0.0 else 1.0
            alpha_step = 1.0 - fade if fade > 0.0 else -1.0 - fade
            alpha_step /= len(spikes[u])
            if len(spikes[u]) == 1:
                alpha = 1.0

            for s in spikes[u]:
                if s.waveform is None or s.sampling_rate is None:
                    raise SpykeException(
                        'Cannot create waveform plot: '
                        'At least one spike has no '
                        'waveform or sampling rate!')
                x = (sp.arange(s.waveform.shape[0]) /
                     s.sampling_rate).rescale(time_unit)
                x_off = max(x_off, x[-1])
                curve = make.curve(
                    x + offset,
                    s.waveform[:, c].rescale(ref_units), u.name,
                    color=color)

                qcol.setAlphaF(alpha)
                curve.setPen(Qt.QPen(qcol))
                alpha += alpha_step

                plot.add_item(curve)
                progress.step()

        for u in strong:
            color = helper.get_object_color(u)
            for s in strong[u]:
                x = (sp.arange(s.waveform.shape[0]) /
                     s.sampling_rate).rescale(time_unit)
                x_off = max(x_off, x[-1])
                outline = make.curve(
                    x + offset, s.waveform[:, c].rescale(ref_units),
                    color='#000000', linewidth=4)
                curve = make.curve(
                    x + offset, s.waveform[:, c].rescale(ref_units),
                    color=color, linewidth=2)
                plot.add_item(outline)
                plot.add_item(curve)
                progress.step()

        offset += x_off
        if c != channels[-1]:
            plot.add_item(
                make.marker((offset, 0), lambda x, y: '',
                            movable=False, markerstyle='|',
                            color='k', linestyle='-', linewidth=1))

    l = _add_legend(plot, spikes, strong)

    return l


def _add_legend(plot, spikes, strong):
    # Keys from spikes and strong without duplicates in original order
    seen = set()
    indices = [k for k in spikes.keys() + strong.keys()
               if k not in seen and not seen.add(k)]
    legend_items = []

    for u in indices:
        legend_curve = make.curve(
            sp.array([0]), sp.array([0]), u.name,
            color=helper.get_object_color(u), linewidth=2)
        legend_items.append(legend_curve)
        plot.add_item(legend_curve)

    l = make.legend(restrict_items=legend_items)
    plot.add_item(l)

    return l


try:
    import unittest2 as ut
    assert ut  # Suppress pyflakes warning about redefinition of unused ut
except ImportError:
    import unittest as ut

import neo
import quantities as pq
import scipy as sp
import spykeutils.spike_train_generation as stg


class CommonSpikeTrainGeneratorTests(object):
    """ Provides some common test cases which should work for all spike train
    generation functions.
    """

    defaultRate = 10 * pq.Hz
    lowRate = 1 * pq.Hz
    highRate = 10000 * pq.Hz

    def invoke_gen_func(self, rate, **kwargs):
        """ This function is called to generate a spike train by the test cases.
        """
        raise NotImplementedError()

    def test_returns_SpikeTrain_containing_spikes(self):
        st = self.invoke_gen_func(self.defaultRate, t_stop=100 * pq.s)
        self.assertIsInstance(st, neo.SpikeTrain)
        self.assertTrue(st.size > 0)

    def test_exception_without_end_condition(self):
        with self.assertRaises(ValueError):
            self.invoke_gen_func(self.defaultRate, t_stop=None)

    def test_times_limited_by_t_start_and_t_stop(self):
        t_start = 10 * pq.s
        t_stop = 20 * pq.s

        st = self.invoke_gen_func(
            self.defaultRate, t_start=t_start, t_stop=t_stop)
        self.assertTrue(sp.all(t_start < st))
        self.assertTrue(sp.all(st <= t_stop))
        self.assertEqual(t_start, st.t_start)
        self.assertEqual(t_stop, st.t_stop)

    def test_num_spikes_limited_by_max_spike(self):
        max_spikes = 10

        # Use a high rate to provoke more than `max_spike` spikes.
        self.assertTrue(
            max_spikes >= self.invoke_gen_func(
                self.highRate, max_spikes=max_spikes).size)

        # Use a long trial length to provoke more than `max_spike` spikes.
        self.assertTrue(
            max_spikes >= self.invoke_gen_func(
                self.lowRate, t_stop=10000 * pq.s, max_spikes=max_spikes).size)

    def test_respects_refractory_period(self):
        refractory = 100 * pq.ms
        st = self.invoke_gen_func(
            self.highRate, max_spikes=1000, refractory=refractory)
        self.assertGreater(
            sp.amax(sp.absolute(sp.diff(st.rescale(pq.s).magnitude))),
            refractory.rescale(pq.s).magnitude)
        st = self.invoke_gen_func(
            self.highRate, t_stop=10 * pq.s, refractory=refractory)
        self.assertGreater(
            sp.amax(sp.absolute(sp.diff(st.rescale(pq.s).magnitude))),
            refractory.rescale(pq.s).magnitude)


class Test_gen_homogeneous_poisson(ut.TestCase, CommonSpikeTrainGeneratorTests):
    def invoke_gen_func(self, rate, **kwargs):
        return stg.gen_homogeneous_poisson(rate, **kwargs)


class Test_gen_inhomogeneous_poisson(
        ut.TestCase, CommonSpikeTrainGeneratorTests):
    def invoke_gen_func(self, max_rate, **kwargs):
        modulation = lambda ts: sp.sin(ts / (5.0 * pq.s) * sp.pi)
        return stg.gen_inhomogeneous_poisson(modulation, max_rate, **kwargs)


if __name__ == '__main__':
    ut.main()



import neo

class DataProvider(object):
    """ Defines all methods that should be implemented by a
    selection/data provider class.

    A `DataProvider` encapsulates
    access to a selection of data. It can be used by plugins to
    acesss data currently selected in the GUI or in saved selections.
    It also contains an attribute `progress`, a
    :class:`spykeutils.progress_indicator.ProgressIndicator` that
    can be used to report the progress of an operation (and is used
    by methods of this class if they can lead to processing times
    of half a second or more).

    This class serves as an abstract base class and should not be
    instantiated."""
    _factories = {}
    no_unit = neo.Unit(name='No Unit')
    no_segment = neo.Segment(name='No segment')
    no_channel = neo.RecordingChannel(name='No recording channel')
    no_channelgroup = neo.RecordingChannelGroup(name='No recording channel group')
    no_unit.annotate(unique_id=-1)
    no_segment.annotate(unique_id=-1)
    no_channel.annotate(unique_id=-1)
    no_channelgroup.annotate(unique_id=-1)

    def __init__(self, name, progress):
        self.name = name
        self.progress = progress

    def _invert_indices(self, dictionary):
        """ Invert the indices of a dictionary of dictionaries.
        """
        dict_type = type(dictionary)
        ret = dict_type()
        for i1 in dictionary:
            for i2 in dictionary[i1]:
                if not i2 in ret:
                    ret[i2] = dict_type()
                ret[i2][i1] = dictionary[i1][i2]
        return ret

    def blocks(self):
        """ Return a list of selected Block objects.

        The returned objects will contain all regular references, not just to
        selected objects.
        """
        return []

    def segments(self):
        """ Return a list of selected Segment objects.

        The returned objects will contain all regular references, not just to
        selected objects.
        """
        return []

    def recording_channel_groups(self):
        """ Return a list of selected RecordingChannelGroup objects.

        The returned objects will contain all regular references, not just to
        selected objects.
        """
        return []

    def recording_channels(self):
        """ Return a list of selected RecordingChannel objects.

        The returned objects will contain all regular references, not just to
        selected objects.
        """
        return []

    def units(self):
        """ Return a list of selected Unit objects.

        The returned objects will contain all regular references, not just to
        selected objects.
        """
        return []

    def selection_blocks(self):
        """ Return a list of selected blocks.

        The returned blocks will contain references to all other selected
        elements further down in the object hierarchy, but no references to
        elements which are not selected. The returned hierarchy is a copy,
        so changes made to it will not persist. The main purpose
        of this function is to provide an object hierarchy that can be
        saved to a neo file. It is not recommended to use it for data
        processing, the respective functions that return objects lower
        in the hierarchy are better suited for that purpose.
        """
        return []

    def spike_trains(self):
        """ Return a list of :class:`neo.core.SpikeTrain` objects.
        """
        return []

    def spike_trains_by_unit(self):
        """ Return a dictionary (indexed by Unit) of lists of
        :class:`neo.core.SpikeTrain` objects.

        If spike trains not attached to a Unit are selected, their
        dicionary key will be ``DataProvider.no_unit``.
        """
        return {}

    def spike_trains_by_segment(self):
        """ Return a dictionary (indexed by Segment) of lists of
        :class:`neo.core.SpikeTrain` objects.

        If spike trains not attached to a Segment are selected, their
        dictionary key will be ``DataProvider.no_segment``.
        """
        return {}

    def spike_trains_by_unit_and_segment(self):
        """ Return a dictionary (indexed by Unit) of dictionaries
        (indexed by Segment) of :class:`neo.core.SpikeTrain` objects.

        If there are multiple spike trains in one Segment for the same Unit,
        only the first will be contained in the returned dictionary. If spike
        trains not attached to a Unit or Segment are selected, their
        dictionary key will be ``DataProvider.no_unit`` or
        ``DataProvider.no_segment``, respectively.
        """
        return {}

    def spike_trains_by_segment_and_unit(self):
        """ Return a dictionary (indexed by Unit) of dictionaries
        (indexed by Segment) of :class:`neo.core.SpikeTrain` objects.

        If there are multiple spike trains in one Segment for the same Unit,
        only the first will be contained in the returned dictionary. If spike
        trains not attached to a Unit or Segment are selected, their
        dictionary key will be ``DataProvider.no_unit`` or
        ``DataProvider.no_segment``, respectively.
        """
        return self._invert_indices(self.spike_trains_by_unit_and_segment())

    def spikes(self):
        """ Return a list of :class:`neo.core.Spike` objects.
        """
        return []

    def spikes_by_unit(self):
        """ Return a dictionary (indexed by Unit) of lists of
        :class:`neo.core.Spike` objects.

        If spikes not attached to a Unit are selected, their
        dicionary key will be ``DataProvider.no_unit``.
        """
        return {}

    def spikes_by_segment(self):
        """ Return a dictionary (indexed by Segment) of lists of
        :class:`neo.core.Spike` objects.

        If spikes not attached to a Segment are selected, their
        dictionary key will be ``DataProvider.no_segment``.
        """
        return {}

    def spikes_by_unit_and_segment(self):
        """ Return a dictionary (indexed by Unit) of dictionaries
        (indexed by Segment) of :class:`neo.core.Spike` lists.

        If there are multiple spikes in one Segment for the same Unit,
        only the first will be contained in the returned dictionary. If
        spikes not attached to a Unit or Segment are selected, their
        dictionary key will be ``DataProvider.no_unit`` or
        ``DataProvider.no_segment``, respectively.
        """
        return {}

    def spikes_by_segment_and_unit(self):
        """ Return a dictionary (indexed by Segment) of dictionaries
        (indexed by Unit) of lists of :class:`neo.core.Spike` lists.

        If spikes not attached to a Unit or Segment are selected, their
        dictionary key will be ``DataProvider.no_unit`` or
        ``DataProvider.no_segment``, respectively.
        """
        return self._invert_indices(self.spikes_by_unit_and_segment())

    def events(self, include_array_events = True):
        """ Return a dictionary (indexed by Segment) of lists of
        Event objects.

        :param bool include_array_events: Determines if EventArray objects
            should be converted to Event objects and included in the returned
            list.
        """
        return {}

    def labeled_events(self, label, include_array_events = True):
        """ Return a dictionary (indexed by Segment) of lists of Event
        objects with the given label.

        :param str label: The name of the Event objects to be returnded
        :param bool include_array_events: Determines if EventArray objects
            should be converted to Event objects and included in the returned
            list.
        """
        return []

    def event_arrays(self):
        """ Return a dictionary (indexed by Segment) of lists of
        EventArray objects.
        """
        return {}

    def epochs(self, include_array_epochs = True):
        """ Return a dictionary (indexed by Segment) of lists of
        Epoch objects.

        :param bool include_array_epochs: Determines if EpochArray objects
            should be converted to Epoch objects and included in the returned
            list.
        """
        return {}

    def labeled_epochs(self, label, include_array_epochs = True):
        """ Return a dictionary (indexed by Segment) of lists of Epoch
        objects with the given label.

        :param str label: The name of the Epoch objects to be returnded
        :param bool include_array_epochs: Determines if EpochArray objects
            should be converted to Epoch objects and included in the returned
            list.
        """
        return []

    def epoch_arrays(self):
        """ Return a dictionary (indexed by Segment) of lists of
        EpochArray objects.
        """
        return {}

    def analog_signals(self, conversion_mode=1):
        """ Return a list of :class:`neo.core.AnalogSignal` objects.

        :param int conversion_mode: Determines what signals are returned:

            1. AnalogSignal objects only
            2. AnalogSignal objects extracted from AnalogSignalArrays only
            3. Both AnalogSignal objects and extracted AnalogSignalArrays
        """
        return []

    def analog_signals_by_segment(self, conversion_mode=1):
        """ Return a dictionary (indexed by Segment) of lists of
        :class:`neo.core.AnalogSignal` objects.

        If analog signals not attached to a Segment are selected, their
        dictionary key will be ``DataProvider.no_segment``.

        :param int conversion_mode: Determines what signals are returned:

            1. AnalogSignal objects only
            2. AnalogSignal objects extracted from AnalogSignalArrays only
            3. Both AnalogSignal objects and extracted AnalogSignalArrays
        """
        return {}

    def analog_signals_by_channel(self, conversion_mode=1):
        """ Return a dictionary (indexed by RecordingChannel) of lists
        of :class:`neo.core.AnalogSignal` objects.

        If analog signals not attached to a RecordingChannel are selected,
        their dictionary key will be ``DataProvider.no_channel``.

        :param int conversion_mode: Determines what signals are returned:

            1. AnalogSignal objects only
            2. AnalogSignal objects extracted from AnalogSignalArrays only
            3. Both AnalogSignal objects and extracted AnalogSignalArrays
        """
        return {}

    def analog_signals_by_channel_and_segment(self, conversion_mode=1):
        """ Return a dictionary (indexed by RecordingChannel) of
        dictionaries (indexed by Segment) of :class:`neo.core.AnalogSignal`
        lists.

        If analog signals not attached to a Segment or
        RecordingChannel are selected, their dictionary key will be
        ``DataProvider.no_segment`` or ``DataProvider.no_channel``,
        respectively.

        :param int conversion_mode: Determines what signals are returned:

            1. AnalogSignal objects only
            2. AnalogSignal objects extracted from AnalogSignalArrays only
            3. Both AnalogSignal objects and extracted AnalogSignalArrays
        """
        return {}

    def analog_signals_by_segment_and_channel(self, conversion_mode=1):
        """ Return a dictionary (indexed by Segment) of
        dictionaries (indexed by RecordingChannel) of :class:`neo.core.AnalogSignal`
        lists.

        If analog signals not attached to a Segment or
        RecordingChannel are selected, their dictionary key will be
        ``DataProvider.no_segment`` or ``DataProvider.no_channel``,
        respectively.

        :param int conversion_mode: Determines what signals are returned:

            1. AnalogSignal objects only
            2. AnalogSignal objects extracted from AnalogSignalArrays only
            3. Both AnalogSignal objects and extracted AnalogSignalArrays
        """
        return self._invert_indices(
            self.analog_signals_by_channel_and_segment(conversion_mode))

    def analog_signal_arrays(self):
        """ Return a list of :class:`neo.core.AnalogSignalArray` objects.
        """
        return []

    def analog_signal_arrays_by_segment(self):
        """ Return a dictionary (indexed by Segment) of lists of
        :class:`neo.core.AnalogSignalArray` objects.

        If analog signals arrays not attached to a Segment are selected,
        their dictionary key will be ``DataProvider.no_segment``.
        """
        return {}

    def analog_signal_arrays_by_channelgroup(self):
        """ Return a dictionary (indexed by RecordingChannelGroup) of
        lists of :class:`neo.core.AnalogSignalArray` objects.

        If analog signals arrays not attached to a RecordingChannel are
        selected, their dictionary key will be
        ``DataProvider.no_channelgroup``.
        """
        return {}

    def analog_signal_arrays_by_channelgroup_and_segment(self):
        """ Return a dictionary (indexed by RecordingChannelGroup) of
        dictionaries (indexed by Segment) of
        :class:`neo.core.AnalogSignalArray` objects.

        If there are multiple analog signals in one RecordingChannel for
        the same Segment, only the first will be contained in the returned
        dictionary. If analog signal arrays not attached to a Segment or
        RecordingChannelGroup are selected, their dictionary key will be
        ``DataProvider.no_segment`` or ``DataProvider.no_channelgroup``,
        respectively.
        """
        return {}

    def analog_signal_arrays_by_segment_and_channelgroup(self):
        """ Return a dictionary (indexed by RecordingChannelGroup) of
        dictionaries (indexed by Segment) of
        :class:`neo.core.AnalogSignalArray` objects.

        If there are multiple analog signals in one RecordingChannel for
        the same Segment, only the first will be contained in the returned
        dictionary. If analog signal arrays not attached to a Segment or
        RecordingChannelGroup are selected, their dictionary key will be
        ``DataProvider.no_segment`` or ``DataProvider.no_channelgroup``,
        respectively.
        """
        return self._invert_indices(
            self.analog_signal_arrays_by_channelgroup_and_segment())

    def refresh_view(self):
        """ Refresh associated views of the data.

        Use this method if when you change the neo hierarchy on which the
        selection is based (e.g. adding or removing objects). It will ensure
        that all current views on the data are updated, for example in
        Spyke Viewer.
        """
        pass

    def data_dict(self):
        """ Return a dictionary with all information to serialize the
        object.
        """
        return {}

    @classmethod
    def from_data(cls, data, progress=None):
        """ Create a new `DataProvider` object from a dictionary. This
        method is mostly for internal use.

        The respective type of `DataProvider` (e.g.
        :class:`spykeviewer.plugin_framework.data_provider_neo.DataProviderNeo`
        has to be imported in the environment where this function is
        called.

        :param dict data: A dictionary containing data from a `DataProvider`
            object, as returned by :func:`data_dict`.
        :param ProgressIndicator progress: The object where loading progress
            will be indicated.
        """
        if progress:
            return cls._factories[data['type']](data, progress)
        return cls._factories[data['type']](data)


import scipy as sp
import quantities as pq

from guiqwt.builder import make
from guiqwt.baseplot import BasePlot
from guiqwt.plot import BaseCurveWidget

from .. import rate_estimation
from ..progress_indicator import ProgressIndicator
from .. import SpykeException
from dialog import PlotDialog
import helper


@helper.needs_qt
def psth(trains, events=None, start=0 * pq.ms, stop=None,
         bin_size=100 * pq.ms, rate_correction=True, bar_plot=False,
         time_unit=pq.ms, progress=None):
    """ Create a peri stimulus time histogram.

    The peri stimulus time histogram gives an estimate of the instantaneous
    rate.

    :param dict trains: A dictionary of :class:`neo.core.SpikeTrain` lists.
    :param dict events: A dictionary of Event objects, indexed by segment.
        The events will be at time 0 on the plot. If None, spike trains
        are used unmodified.
    :param start: The desired time for the start of the first bin. It
        will be recalculated if there are spike trains which start later
        than this time. This parameter can be negative (which could be
        useful when aligning on events).
    :type start: Quantity scalar
    :param stop: The desired time for the end of the last bin. It will
        be recalculated if there are spike trains which end earlier
        than this time.
    :type stop: Quantity scalar
    :param bin_size: The bin size for the histogram.
    :type bin_size: Quantity scalar
    :param bool rate_correction: Determines if a rates (``True``) or
        counts (``False``) are shown.
    :param bool bar_plot: Determines if a bar plot (``True``) or a line
        plot (``False``) will be created. In case of a bar plot, one plot
        for each index in ``trains`` will be created.
    :param Quantity time_unit: Unit of X-Axis.
    :param progress: Set this parameter to report progress.
    :type progress: :class:`spykeutils.progress_indicator.ProgressIndicator`
    """
    if not trains:
        raise SpykeException('No spike trains for PSTH!')
    if not progress:
        progress = ProgressIndicator()

    # Align spike trains
    for u in trains:
        if events:
            trains[u] = rate_estimation.aligned_spike_trains(
                trains[u], events)

    rates, bins = rate_estimation.psth(
        trains, bin_size, start=start, stop=stop,
        rate_correction=rate_correction)
    bins = bins.rescale(time_unit)

    if not psth:
        raise SpykeException('No spike trains for PSTH!')

    win_title = 'PSTH | Bin size %.2f %s' % (bin_size,
                                             time_unit.dimensionality.string)
    win = PlotDialog(toolbar=True, wintitle=win_title, min_plot_width=150,
                     min_plot_height=100)

    legends = []
    if bar_plot:
        ind = 0
        columns = int(sp.sqrt(len(rates)))
        for i, r in rates.iteritems():
            if i and hasattr(i, 'name') and i.name:
                name = i.name
            else:
                name = 'Unknown'

            pW = BaseCurveWidget(win)
            plot = pW.plot

            show_rates = list(r)
            show_rates.insert(0, show_rates[0])
            curve = make.curve(
                bins, show_rates, name, color='k',
                curvestyle="Steps", shade=1.0)
            plot.add_item(curve)

            # Create legend
            color = helper.get_object_color(i)
            color_curve = make.curve(
                [], [], name, color, 'NoPen', linewidth=1, marker='Rect',
                markerfacecolor=color, markeredgecolor=color)
            plot.add_item(color_curve)
            legends.append(make.legend(restrict_items=[color_curve]))
            plot.add_item(legends[-1])

            # Prepare plot
            plot.set_antialiasing(False)
            scale = plot.axisScaleDiv(BasePlot.Y_LEFT)
            plot.setAxisScale(BasePlot.Y_LEFT, 0, scale.upperBound())
            if ind % columns == 0:
                if not rate_correction:
                    plot.set_axis_title(BasePlot.Y_LEFT, 'Spike Count')
                else:
                    plot.set_axis_title(BasePlot.Y_LEFT, 'Rate')
                    plot.set_axis_unit(BasePlot.Y_LEFT, 'Hz')
            if ind >= len(trains) - columns:
                plot.set_axis_title(BasePlot.X_BOTTOM, 'Time')
                plot.set_axis_unit(
                    BasePlot.X_BOTTOM, time_unit.dimensionality.string)

            win.add_plot_widget(pW, ind, column=ind % columns)
            ind += 1
    else:
        bins = 0.5 * sp.diff(bins) + bins[:-1]

        pW = BaseCurveWidget(win)
        plot = pW.plot
        legend_items = []

        for i, r in rates.iteritems():
            if i and hasattr(i, 'name') and i.name:
                name = i.name
            else:
                name = 'Unknown'

            curve = make.curve(
                bins, r, name,
                color=helper.get_object_color(i))
            legend_items.append(curve)
            plot.add_item(curve)

        win.add_plot_widget(pW, 0)

        legends.append(make.legend(restrict_items=legend_items))
        plot.add_item(legends[-1])

        if not rate_correction:
            plot.set_axis_title(BasePlot.Y_LEFT, 'Spike Count')
        else:
            plot.set_axis_title(BasePlot.Y_LEFT, 'Rate')
            plot.set_axis_unit(BasePlot.Y_LEFT, 'Hz')
        plot.set_axis_title(BasePlot.X_BOTTOM, 'Time')
        plot.set_axis_unit(BasePlot.X_BOTTOM, time_unit.dimensionality.string)
        plot.set_antialiasing(True)

    win.add_custom_curve_tools()
    win.add_legend_option(legends, True)
    progress.done()
    win.show()

    if bar_plot and len(rates) > 1:
        win.add_x_synchronization_option(True, range(len(rates)))
        win.add_y_synchronization_option(False, range(len(rates)))

    return win


import traceback
import inspect
import neo
from neo.io.baseio import BaseIO

from .. import SpykeException


def load_from_file(path):
    """ Load IO plugins from a Python file. Inserts
    the loaded plugins into the neo.iolist.

    :param str path: The path to the file to search for IO plugins.
    """
    f = open(path)
    load_from_string(f.read(), path)


def load_from_string(code, path='<string>'):
    """ Load IO plugins from Python code. Inserts
    the loaded plugins into the neo.iolist.

    :param str code: The IO plugin code.
    :param str path: The path for the IO plugin.
    """
    exc_globals = {}
    try:
        exec(code, exc_globals)
    except Exception:
        raise SpykeException('Error during execution of ' +
                             'potential Neo IO file ' + path + ':\n' +
                             traceback.format_exc() + '\n')

    for cl in exc_globals.values():
        if not inspect.isclass(cl):
            continue

        # Should be a subclass of BaseIO...
        if not issubclass(cl, BaseIO):
            continue
            # but should not be BaseIO (can happen when directly imported)
        if cl == BaseIO:
            continue

        cl._is_spyke_plugin = True
        cl._python_file = path
        neo.io.iolist.insert(0, cl)


from guiqwt.builder import make
from guiqwt.baseplot import BasePlot
from guiqwt.plot import BaseCurveWidget

import quantities as pq

from .. import SpykeException
from dialog import PlotDialog
import helper


@helper.needs_qt
def raster(trains, time_unit=pq.ms, show_lines=True, events=None, epochs=None):
    """ Create a new plotting window with a rasterplot of spiketrains.

        :param dict trains: Dictionary of spike trains indexed by a
            Neo object (Unit or Segment).
        :param Quantity time_unit: Unit of X-Axis.
        :param bool show_lines: Determines if a horizontal line will be shown
            for each spike train.
        :param sequence events: A sequence of neo `Event` objects that will
            be marked on the plot.

    """
    if not trains:
        raise SpykeException('No spike trains for rasterplot')

    if not time_unit:
        time_unit = pq.ms

    win_title = 'Spike Trains'
    win = PlotDialog(toolbar=True, wintitle=win_title, major_grid=False)

    pW = BaseCurveWidget(win)
    plot = pW.plot

    if events is None:
        events = []
    if epochs is None:
        epochs = []

    offset = len(trains)
    legend_items = []
    for u, t in trains.iteritems():
        color = helper.get_object_color(u)

        train = helper.add_spikes(
            plot, t, color, 2, 21, offset, u.name, time_unit)

        if u.name:
            legend_items.append(train)
        if show_lines:
            plot.add_item(make.curve(
                [t.t_start.rescale(time_unit), t.t_stop.rescale(time_unit)],
                [offset, offset], color='k'))
        offset -= 1

    helper.add_epochs(plot, epochs, time_unit)
    helper.add_events(plot, events, time_unit)

    plot.set_axis_title(BasePlot.X_BOTTOM, 'Time')
    plot.set_axis_unit(BasePlot.X_BOTTOM, time_unit.dimensionality.string)

    win.add_plot_widget(pW, 0)

    legend = make.legend(restrict_items=legend_items)
    plot.add_item(legend)
    win.add_legend_option([legend], True)

    if len(trains) > 1:
        plot.set_axis_limits(BasePlot.Y_LEFT, 0.5, len(trains) + 0.5)

    win.add_custom_curve_tools()
    win.show()

    return win


import scipy as sp
import quantities as pq


# At least up to quantities 0.10.1 the scipy element-wise minimum and maximum
# function did not work.
# This has been reported upstream as issue #53:
# <https://github.com/python-quantities/python-quantities/issues/53>

def _fix_binary_scipy_function_with_out_param(f):
    def _fixed(x1, x2, out=None):
        if isinstance(x1, pq.Quantity) or isinstance(x2, pq.Quantity):
            x1 = x1 * pq.dimensionless
            x2 = x2 * pq.dimensionless
            if out is None:
                x2 = x2.rescale(x1.units)
                return f(x1.magnitude, x2.magnitude) * x1.units
            else:
                x1 = x1.rescale(out.units)
                x2 = x2.rescale(out.units)
                f(x1.magnitude, x2.magnitude, out.magnitude)
                return out
        else:
            return f(x1, x2, out)
    return _fixed
minimum = _fix_binary_scipy_function_with_out_param(sp.minimum)
maximum = _fix_binary_scipy_function_with_out_param(sp.maximum)


# At least up to quantities 0.10.1 the scipy meshgrid  and concatenate
# functions did lose units.
# This has been reported upstream as issue #47:
# <https://github.com/python-quantities/python-quantities/issues/47>
def _fix_scipy_meshgrid(f):
    def _fixed(x, y):
        rx, ry = f(x, y)
        if isinstance(x, pq.Quantity) and not isinstance(rx, pq.Quantity):
            rx = rx * x.units
        if isinstance(y, pq.Quantity) and not isinstance(ry, pq.Quantity):
            ry = ry * y.units
        return rx, ry
    return _fixed
meshgrid = _fix_scipy_meshgrid(sp.meshgrid)


def _fix_scipy_concatenate(f):
    def _fixed(arrays, axis=0):
        is_quantity = len(arrays) > 0 and isinstance(arrays[0], pq.Quantity)
        if is_quantity:
            arrays = [(a * pq.dimensionless).rescale(arrays[0].units)
                      for a in arrays]
        else:
            for a in arrays:
                if (isinstance(a, pq.Quantity) and
                        a.units.simplified != pq.dimensionless):
                    raise ValueError(
                        'Cannot concatenate arrays of different units')
        concatenated = f(arrays, axis=axis)
        if is_quantity:
            concatenated = concatenated * arrays[0].units
        return concatenated
    return _fixed
concatenate = _fix_scipy_concatenate(sp.concatenate)


# At least up to quantities 0.10.1 the scipy inner and diag functions did not
# respect units.
def _fix_binary_scipy_function(f):
    def _fixed(x1, x2):
        if isinstance(x1, pq.Quantity) or isinstance(x2, pq.Quantity):
            x1 = x1 * pq.dimensionless
            x2 = x2 * pq.dimensionless
            x2 = x2.rescale(x1.units)
            return f(x1.magnitude, x2.magnitude) * x1.units
        else:
            return f(x1, x2)
    return _fixed
inner = _fix_binary_scipy_function(sp.inner)


def diag(v, k=0):
    if isinstance(v, pq.Quantity):
        return sp.diag(v, k) * v.units
    else:
        return sp.diag(v, k)


# linspace loses unit for only one bin, see bug report
# <https://github.com/python-quantities/python-quantities/issues/55>
def linspace(start, stop, num=50, endpoint=True, retstep=False):
    if int(num) == 1 and isinstance(start, pq.Quantity):
        return sp.linspace(start, stop, num, endpoint, retstep) * start.units
    else:
        return sp.linspace(start, stop, num, endpoint, retstep)



import scipy as sp
import neo

from . import SpykeException


def spike_train_to_spikes(spike_train, include_waveforms=True):
    """ Return a list of spikes for a spike train.

    Note that while the created spikes have references to the same segment and
    unit as the spike train, the relationships in the other direction are
    not automatically created (the spikes are not attached to the unit or
    segment). Other properties like annotations are not copied or referenced
    in the created spikes.

    :param spike_train: A spike train from which the :class:`neo.core.Spike`
        objects are constructed.
    :type spike_train: :class:`neo.core.SpikeTrain`
    :param bool include_waveforms: Determines if the ``waveforms`` property is
        converted to the spike waveforms. If ``waveforms`` is None, this
        parameter has no effect.
    :returns: A list of :class:`neo.core.Spike` objects, one for every
        spike in ``spike_train``.
    :rtype: list
    """
    waves = None
    if include_waveforms:
        waves = spike_train.waveforms

    spikes = []
    for i, t in enumerate(spike_train):
        s = neo.Spike(t, sampling_rate=spike_train.sampling_rate,
                      left_sweep=spike_train.left_sweep)
        if waves is not None:
            s.waveform = waves[i, :, :]
        s.unit = spike_train.unit
        s.segment = spike_train.segment
        spikes.append(s)

    return spikes


def spikes_to_spike_train(spikes, include_waveforms=True):
    """ Return a spike train for a list of spikes.

    All spikes must have an identical left sweep, the same unit and the same
    segment, otherwise a ``SpykeException`` is raised.

    Note that while the created spike train has references to the same
    segment and unit as the spikes, the relationships in the other direction
    are not automatically created (the spike train is not attached to the
    unit or segment). Other properties like annotations are not copied or
    referenced in the created spike train.

    :param sequence spikes: A sequence of :class:`neo.core.Spike` objects
        from which the spike train is constructed.
    :param bool include_waveforms: Determines if the waveforms from the spike
        objects are used to fill the ``waveforms`` property of the resulting
        spike train. If ``True``, all spikes need a ``waveform`` property
        with the same shape or a ``SpykeException`` is raised (or the
        ``waveform`` property needs to be ``None`` for all spikes).
    :return: All elements of ``spikes`` as spike train.
    :rtype: :class:`neo.core.SpikeTrain`
    """
    if not spikes:
        raise SpykeException('No spikes to create spike train!')

    tu = spikes[0].time.units
    times = sp.zeros(len(spikes)) * tu
    s = spikes[0].segment
    u = spikes[0].unit
    ls = spikes[0].left_sweep

    if include_waveforms and spikes[0].waveform is not None:
        sh = spikes[0].waveform.shape
        wu = spikes[0].waveform.units
        waves = sp.zeros((len(spikes), sh[0], sh[1])) * wu
    else:
        waves = None
        sh = None

    for i, spike in enumerate(spikes):
        if (u != spike.unit or s != spike.segment or
                ls != spike.left_sweep):
            raise SpykeException('Cannot create spike train from spikes with '
                                 'nonuniform properties!')

        times[i] = spikes[i].time

        if include_waveforms:
            if spike.waveform is None:
                if waves is not None:
                    raise SpykeException('Cannot create spike train from '
                                         'spikes where some waveforms are '
                                         'None')
            elif sh != spike.waveform.shape:
                raise SpykeException('Cannot create spike train from spikes '
                                     'with nonuniform waveform shapes!')
            if waves is not None:
                waves[i, :, :] = spike.waveform

    ret = neo.SpikeTrain(times, t_start=times.min(), t_stop=times.max(),
                         waveforms=waves, left_sweep=ls)
    ret.unit = u
    ret.segment = s
    ret.left_sweep = ls
    return ret


def analog_signal_array_to_analog_signals(signal_array):
    """ Return a list of analog signals for an analog signal array.

    If ``signal_array`` is attached to a recording channel group with exactly
    is many channels as there are channels in ``signal_array``, each created
    signal will be assigned the corresponding channel. If the attached
    recording channel group has only one recording channel, all created signals
    will be assigned to this channel. In all other cases, the created
    signal will not have a reference to a recording channel.

    Note that while the created signals may have references to a segment and
    channels, the relationships in the other direction are
    not automatically created (the signals are not attached to the recording
    channel or segment). Other properties like annotations are not copied or
    referenced in the created analog signals.

    :param signal_array: An analog signal array from which the
        :class:`neo.core.AnalogSignal` objects are constructed.
    :type signal_array: :class:`neo.core.AnalogSignalArray`
    :return: A list of analog signals, one for every channel in
        ``signal_array``.
    :rtype: list
    """
    signals = []
    rcg = signal_array.recordingchannelgroup

    for i in xrange(signal_array.shape[1]):
        s = neo.AnalogSignal(
            signal_array[:, i], t_start=signal_array.t_start,
            sampling_rate=signal_array.sampling_rate)
        if len(rcg.recordingchannels) == 1:
            s.recordingchannel = rcg.recordingchannels[0]
        elif len(rcg.recordingchannels) == signal_array.shape[1]:
            s.recordingchannel = rcg.recordingchannels[i]
        s.segment = signal_array.segment
        signals.append(s)

    return signals


def event_array_to_events(event_array):
    """ Return a list of events for an event array.

    Note that while the created events may have references to a segment,
    the relationships in the other direction are not automatically created
    (the events are not attached to the segment). Other properties like
    annotations are not copied or referenced in the created events.

    :param event_array: An event array from which the Event objects are
        constructed.
    :type event_array: :class:`neo.core.EventArray`
    :return: A list of events, one for of the events in ``event_array``.
    :rtype: list
    """
    events = []
    for i, t in enumerate(event_array.times):
        e = neo.Event(
            t, event_array.labels[i] if i < len(event_array.labels) else '')
        e.segment = event_array.segment
        events.append(e)
    return events


def epoch_array_to_epochs(epoch_array):
    """ Return a list of epochs for an epoch array.

    Note that while the created epochs may have references to a segment,
    the relationships in the other direction are not automatically created
    (the events are not attached to the segment). Other properties like
    annotations are not copied or referenced in the created epochs.

    :param epoch_array: A period array from which the Epoch objects are
        constructed.
    :type epoch_array: :class:`neo.core.EpochArray`
    :return: A list of events, one for of the events in ``epoch_array``.
    :rtype: list
    """
    periods = []
    for i, t in enumerate(epoch_array.times):
        p = neo.Epoch(
            t, epoch_array.durations[i],
            epoch_array.labels[i] if i < len(epoch_array.labels) else '')
        p.segment = epoch_array.segment
        periods.append(p)
    return periods



import json
from data_provider import DataProvider
from data_provider_neo import NeoDataProvider
from ..progress_indicator import ProgressIndicator


class NeoStoredProvider(NeoDataProvider):
    def __init__(self, data, progress=ProgressIndicator()):
        super(NeoStoredProvider, self).__init__(data['name'], progress)
        self.data = data
        self.block_cache = None

    @classmethod
    def from_current_selection(cls, name, viewer):
        """ Create new NeoStoredProvider from current viewer selection
        """
        data = cls._get_data_from_viewer(viewer)
        data['name'] = name
        return cls(data, viewer.progress)

    @classmethod
    def from_file(cls, file_name, progress=ProgressIndicator()):
        """ Create new DBStoredProvider from JSON file
        """
        data = json.load(file_name)
        return cls(data, progress)

    def save(self, file_name):
        """ Save selection to JSON file
        """
        f = open(file_name, 'w')
        json.dump(self.data, f, sort_keys=True, indent=4)
        f.close()

    def data_dict(self):
        """ Return a dictionary with all information to serialize the object
        """
        self.data['name'] = self.name
        return self.data

    def blocks(self):
        """ Return a list of selected Block objects
        """
        if self.block_cache is None:
            self.block_cache = []
            for b in self.data['blocks']:
                cl = None
                rp = None
                if len(b) > 2:
                    cl = self.find_io_class(b[2])
                if len(b) > 3:
                    rp = b[3]
                self.block_cache.append(NeoDataProvider.get_block(
                    b[1], b[0], force_io=cl, read_params=rp))
        return self.block_cache

    def segments(self):
        """ Return a list of selected Segment objects
        """
        blocks = self.blocks()
        segments = []
        for s in self.data['segments']:
            segments.append(blocks[s[1]].segments[s[0]])
        return segments

    def recording_channel_groups(self):
        """ Return a list of selected
        """
        blocks = self.blocks()
        rcgs = []
        for rcg in self.data['channel_groups']:
            rcgs.append(blocks[rcg[1]].recordingchannelgroups[rcg[0]])
        return rcgs

    def recording_channels(self):
        """ Return a list of selected recording channel indices
        """
        rcgs = self.recording_channel_groups()
        rcs = []
        for rc in self.data['channels']:
            rcs.append(rcgs[rc[1]].recordingchannels[rc[0]])
        return rcs

    def units(self):
        """ Return a list of selected Unit objects
        """
        rcgs = self.recording_channel_groups()
        units = []
        for u in self.data['units']:
            units.append(rcgs[u[1]].units[u[0]])
        return units

# Enable automatic creation of NeoStoredProvider objects
DataProvider._factories['Neo'] = NeoStoredProvider



try:
    import unittest2 as ut
    assert ut  # Suppress pyflakes warning about redefinition of unused ut
except ImportError:
    import unittest as ut

from builders import create_empty_spike_train
from mock import MagicMock
from numpy.testing import assert_array_almost_equal, assert_array_equal
import neo
import quantities as pq
import scipy as sp
import spykeutils.signal_processing as sigproc


class TestKernel(ut.TestCase):
    def test_call_returns_result_of_evaluate(self):
        kernel_size = 1.3
        kernel = sigproc.Kernel(kernel_size, normalize=False)
        kernel._evaluate = MagicMock(name='_evaluate')
        kernel._evaluate.return_value = 42

        t = 3.3
        actual = kernel(t)

        kernel._evaluate.assert_called_with(t, kernel_size)
        self.assertEqual(kernel._evaluate.return_value, actual)

    def test_call_can_normalize_evaluate(self):
        kernel_size = 1.3
        kernel = sigproc.Kernel(kernel_size, normalize=True)
        kernel._evaluate = MagicMock(name='_evaluate')
        kernel._evaluate.return_value = 42
        kernel.normalization_factor = MagicMock(name='normalization_factor')
        kernel.normalization_factor.return_value = 0.5

        t = 3.3
        actual = kernel(t)

        kernel._evaluate.assert_called_with(t, kernel_size)
        kernel.normalization_factor.assert_called_with(kernel_size)
        self.assertEqual(
            kernel.normalization_factor.return_value *
            kernel._evaluate.return_value, actual)

    def test_call_can_overwrite_kernel_size(self):
        kernel_size = 1.3
        kernel = sigproc.Kernel(5.6, normalize=True)
        kernel._evaluate = MagicMock(name='_evaluate')
        kernel._evaluate.return_value = 42
        kernel.normalization_factor = MagicMock(name='normalization_factor')
        kernel.normalization_factor.return_value = 0.5

        t = 3.3
        actual = kernel(t, kernel_size)

        kernel._evaluate.assert_called_with(t, kernel_size)
        kernel.normalization_factor.assert_called_with(kernel_size)
        self.assertEqual(
            kernel.normalization_factor.return_value *
            kernel._evaluate.return_value, actual)

    def test_summed_dist_matrix(self):
        kernel = sigproc.Kernel(1.0, normalize=False)
        kernel._evaluate = lambda t, _: t
        vectors = [sp.array([2.0, 1.0, 3.0]), sp.array([1.5, 4.0])]
        expected = sp.array([[0.0, -4.5], [4.5, 0.0]])
        actual = kernel.summed_dist_matrix(vectors)
        assert_array_almost_equal(expected, actual)

    def test_summed_dist_matrix_with_units(self):
        kernel = sigproc.Kernel(2000 * pq.ms, normalize=False)
        kernel._evaluate = lambda t, size: 1.0 / size / size * t
        vectors = [sp.array([2.0, 1.0, 3.0]) * pq.s,
                   sp.array([1500, 4000]) * pq.ms]
        expected = sp.array([[0.0, -1.125], [1.125, 0.0]]) / pq.s
        actual = kernel.summed_dist_matrix(vectors)
        assert_array_almost_equal(expected, actual.rescale(1.0 / pq.s))


class TestSymmetricKernel(ut.TestCase):
    def test_summed_dist_matrix(self):
        kernel = sigproc.Kernel(1.0, normalize=False)
        kernel._evaluate = lambda t, _: sp.absolute(t)
        vectors = [sp.array([2.0, 1.0, 3.0]), sp.array([1.5, 4.0])]
        expected = sp.array([[8.0, 8.5], [8.5, 5.0]])
        actual = kernel.summed_dist_matrix(vectors)
        assert_array_almost_equal(expected, actual)

    def test_summed_dist_matrix_with_units(self):
        kernel = sigproc.Kernel(2000 * pq.ms, normalize=False)
        kernel._evaluate = lambda t, size: 1.0 / size / size * sp.absolute(t)
        vectors = [sp.array([2.0, 1.0, 3.0]) * pq.s,
                   sp.array([1500, 4000]) * pq.ms]
        expected = sp.array([[2.0, 2.125], [2.125, 1.25]]) / pq.s
        actual = kernel.summed_dist_matrix(vectors)
        assert_array_almost_equal(expected, actual.rescale(1.0 / pq.s))


class Test_discretize_kernel(ut.TestCase):
    def test_discretizes_requested_area(self):
        kernel = sigproc.Kernel(1.0, normalize=False)
        kernel.boundary_enclosing_at_least = MagicMock(
            name='boundary_enclosing_at_least')
        kernel.boundary_enclosing_at_least.return_value = 2.0
        kernel._evaluate = lambda x, _: sp.ones(len(x))
        sampling_rate = 1.0
        mock_discretization = sp.ones(5)

        kernel_area_fraction = 0.5
        actual = sigproc.discretize_kernel(
            kernel, sampling_rate, kernel_area_fraction)

        kernel.boundary_enclosing_at_least.assert_called_with(
            kernel_area_fraction)
        assert_array_equal(actual, mock_discretization)

    def test_discretizes_requested_area_with_units(self):
        kernel = sigproc.Kernel(100.0 * pq.ms, normalize=False)
        kernel.boundary_enclosing_at_least = MagicMock(
            name='boundary_enclosing_at_least')
        kernel.boundary_enclosing_at_least.return_value = 100.0 * pq.ms
        kernel._evaluate = lambda x, _: sp.ones(len(x))
        sampling_rate = 10.0 * pq.Hz
        mock_discretization = sp.ones(3)

        kernel_area_fraction = 0.5
        actual = sigproc.discretize_kernel(
            kernel, sampling_rate, kernel_area_fraction)

        kernel.boundary_enclosing_at_least.assert_called_with(
            kernel_area_fraction)
        assert_array_equal(actual, mock_discretization)

    def test_discretizes_requested_number_of_bins(self):
        kernel = sigproc.Kernel(1.0, normalize=False)
        kernel._evaluate = lambda x, _: sp.ones(len(x))
        sampling_rate = 1.0
        num_bins = 23
        mock_discretization = sp.ones(num_bins)

        actual = sigproc.discretize_kernel(
            kernel, sampling_rate, num_bins=num_bins)
        assert_array_equal(actual, mock_discretization)

    def test_can_normalize_to_unit_area(self):
        kernel = sigproc.Kernel(1.0, normalize=False)
        kernel._evaluate = lambda x, _: sp.ones(len(x))
        sampling_rate = 2.0
        num_bins = 20
        mock_discretization = sp.ones(num_bins) / num_bins * sampling_rate

        actual = sigproc.discretize_kernel(
            kernel, sampling_rate, num_bins=num_bins, ensure_unit_area=True)
        assert_array_equal(actual, mock_discretization)


class TestCausalDecayingExpKernel(ut.TestCase):
    def setUp(self):
        self.kernel_size = 500 * pq.ms
        self.kernel = sigproc.CausalDecayingExpKernel(self.kernel_size)

    def test_evaluates_to_correct_values(self):
        t = sp.array([-0.1, 0, 0.6, 1]) * pq.s
        expected = sp.array([0.0, 2.0, 0.60238842, 0.27067057]) / pq.s
        actual = self.kernel(t)
        assert_array_almost_equal(expected, actual.rescale(expected.units))

    def test_boundary_enclosing_at_least_is_correct(self):
        actual = self.kernel.boundary_enclosing_at_least(0.99)
        self.assertAlmostEqual(actual.rescale(pq.s), 2.30258509 * pq.s)


class TestGaussianKernel(ut.TestCase):
    def setUp(self):
        self.kernel_size = 500 * pq.ms
        self.kernel = sigproc.GaussianKernel(self.kernel_size)

    def test_evaluates_to_correct_values(self):
        t = sp.array([-0.1, 0, 0.6, 1]) * pq.s
        expected = sp.array([0.78208539, 0.79788456, 0.38837211, 0.10798193]) /\
            pq.s
        actual = self.kernel(t)
        assert_array_almost_equal(expected, actual.rescale(expected.units))

    def test_boundary_enclosing_at_least_is_correct(self):
        actual = self.kernel.boundary_enclosing_at_least(0.99)
        self.assertAlmostEqual(actual.rescale(pq.s), 1.28791465 * pq.s)


class TestLaplacianKernel(ut.TestCase):
    def setUp(self):
        self.kernel_size = 500 * pq.ms
        self.kernel = sigproc.LaplacianKernel(self.kernel_size)

    def test_evaluates_to_correct_values(self):
        t = sp.array([-0.1, 0, 0.6, 1]) * pq.s
        expected = sp.array([0.81873075, 1.0, 0.30119421, 0.13533528]) /\
            pq.s
        actual = self.kernel(t)
        assert_array_almost_equal(expected, actual.rescale(expected.units))

    def test_boundary_enclosing_at_least_is_correct(self):
        actual = self.kernel.boundary_enclosing_at_least(0.99)
        self.assertAlmostEqual(actual.rescale(pq.s), 2.30258509 * pq.s)

    def test_summed_dist_matrix(self):
        kernel = sigproc.LaplacianKernel(1.0, normalize=False)
        vectors = [sp.array([2.0, 1.0, 3.0]), sp.array([1.5, 4.0])]
        expected = sp.array(
            [[4.7421883311589941, 1.9891932723496157],
             [1.9891932723496157, 2.1641699972477975]])
        actual = kernel.summed_dist_matrix(vectors)
        assert_array_almost_equal(expected, actual)

    def test_summed_dist_matrix_with_units(self):
        kernel = sigproc.LaplacianKernel(1000 * pq.ms, normalize=True)
        vectors = [sp.array([2.0, 1.0, 3.0]) * pq.s,
                   sp.array([1500, 4000]) * pq.ms]
        expected = sp.array(
            [[4.7421883311589941, 1.9891932723496157],
             [1.9891932723496157, 2.1641699972477975]]) / 2.0 / pq.s
        actual = kernel.summed_dist_matrix(vectors)
        assert_array_almost_equal(expected, actual.rescale(1.0 / pq.s))


class TestRectangularKernel(ut.TestCase):
    def setUp(self):
        self.kernel_size = 500 * pq.ms
        self.kernel = sigproc.RectangularKernel(self.kernel_size)

    def test_evaluates_to_correct_values(self):
        t = sp.array([-0.1, 0, 0.6, 1]) * pq.s
        expected = sp.array([1.0, 1.0, 0.0, 0.0]) / pq.s
        actual = self.kernel(t)
        assert_array_almost_equal(expected, actual.rescale(expected.units))

    def test_boundary_enclosing_at_least_is_correct(self):
        actual = self.kernel.boundary_enclosing_at_least(0.99)
        self.assertAlmostEqual(
            actual.rescale(self.kernel_size.units), self.kernel_size)


class TestTriangularKernel(ut.TestCase):
    def setUp(self):
        self.kernel_size = 500 * pq.ms
        self.kernel = sigproc.TriangularKernel(self.kernel_size)

    def test_evaluates_to_correct_values(self):
        t = sp.array([-0.7, -0.1, 0, 0.3, 0.6, 1]) * pq.s
        expected = 2 * sp.array([0.0, 0.8, 1.0, 0.4, 0.0, 0.0]) / pq.s
        actual = self.kernel(t)
        assert_array_almost_equal(expected, actual.rescale(expected.units))

    def test_boundary_enclosing_at_least_is_correct(self):
        actual = self.kernel.boundary_enclosing_at_least(0.99)
        self.assertAlmostEqual(
            actual.rescale(self.kernel_size.units), self.kernel_size)


class Test_smooth(ut.TestCase):
    def test_convolution_with_empty_binned_array_returns_array_of_zeros(self):
        binned = sp.zeros(10)
        sampling_rate = 10 * pq.Hz
        result = sigproc.smooth(binned, sigproc.GaussianKernel(), sampling_rate)
        self.assertTrue(sp.all(result == 0.0))

    def test_length_of_returned_array_equals_length_of_binned(self):
        binned = sp.ones(10)
        sampling_rate = 10 * pq.Hz
        result = sigproc.smooth(binned, sigproc.GaussianKernel(), sampling_rate)
        self.assertEqual(binned.size, result.size)

    def test_returns_smoothed_representation(self):
        binned = sp.array([0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0])
        sampling_rate = 4 * pq.Hz
        kernel = sigproc.RectangularKernel(0.3 * pq.s)
        # Because of the low sampling rate the expected result is a bit off
        # from the analytical result.
        expected = sp.array(
            [0.0, 0.0, 0.0, 1.6666666, 1.6666666, 1.6666666, 0.0,
             1.6666666, 1.6666666, 1.6666666, 0.0, 0.0]) * pq.Hz
        actual = sigproc.smooth(binned, kernel, sampling_rate=sampling_rate)
        assert_array_almost_equal(expected, actual)

    def test_mode_allows_full_convolution(self):
        binned = sp.ones(10)
        sampling_rate = 2.0 * pq.Hz
        kernel = sigproc.RectangularKernel(0.6 * pq.s)
        expected_length = 12
        actual = sigproc.smooth(
            binned, kernel, sampling_rate=sampling_rate, mode='full')
        self.assertEqual(actual.size, expected_length)

    def test_mode_allows_valid_convolution(self):
        binned = sp.ones(10)
        sampling_rate = 2.0 * pq.Hz
        kernel = sigproc.RectangularKernel(0.6 * pq.s)
        expected_length = 8
        actual = sigproc.smooth(
            binned, kernel, sampling_rate=sampling_rate, mode='valid')
        self.assertEqual(actual.size, expected_length)


class Test_st_convolve(ut.TestCase):
    def test_convolution_with_empty_spike_train_returns_array_of_zeros(self):
        st = create_empty_spike_train()
        result, _ = sigproc.st_convolve(st, sigproc.GaussianKernel(), 1 * pq.Hz)
        self.assertTrue(sp.all(result == 0.0))

    def test_length_of_returned_array_equals_sampling_rate_times_duration(self):
        start = 2.0 * pq.s
        stop = 5.0 * pq.s
        duration = stop - start
        sampling_rate = 12 * pq.Hz
        expected_length = (sampling_rate * duration).simplified

        st = create_empty_spike_train(start, stop)
        result, _ = sigproc.st_convolve(
            st, sigproc.GaussianKernel(), sampling_rate=sampling_rate)
        self.assertEqual(expected_length, result.size)

    def test_returns_convolved_spike_train(self):
        st = neo.SpikeTrain(sp.array([1.0, 2.0]) * pq.s, t_stop=3.0 * pq.s)
        kernel = sigproc.RectangularKernel(0.3 * pq.s)
        # Because of the low sampling rate the expected result is a bit off
        # from the analytical result.
        expected = sp.array(
            [0.0, 0.0, 0.0, 1.6666666, 1.6666666, 1.6666666, 0.0,
             1.6666666, 1.6666666, 1.6666666, 0.0, 0.0]) * pq.Hz
        actual, _ = sigproc.st_convolve(st, kernel, sampling_rate=4 * pq.Hz)
        assert_array_almost_equal(expected, actual)

    def test_returns_discretization_bins(self):
        start = 2.0 * pq.s
        stop = 5.0 * pq.s
        sampling_rate = 2.0 * pq.Hz
        expected = sp.array([2.0, 2.5, 3.0, 3.5, 4.0, 4.5, 5.0]) * pq.s

        st = create_empty_spike_train(start, stop)
        _, bins = sigproc.st_convolve(
            st, sigproc.GaussianKernel(), sampling_rate=sampling_rate)
        assert_array_almost_equal(expected, bins)

    def test_mode_allows_full_convolution(self):
        start = 2.0 * pq.s
        stop = 5.0 * pq.s
        sampling_rate = 2.0 * pq.Hz
        kernel = sigproc.RectangularKernel(0.6 * pq.s)
        st = create_empty_spike_train(start, stop)

        expected_length = (stop - start) * sampling_rate + 2
        expected_bins = sp.array(
            [1.5, 2.0, 2.5, 3.0, 3.5, 4.0, 4.5, 5.0, 5.5]) * pq.s

        binned, bins = sigproc.st_convolve(
            st, kernel, mode='full', sampling_rate=sampling_rate)
        self.assertEqual(binned.size, expected_length)
        assert_array_almost_equal(expected_bins, bins)

    def test_mode_allows_valid_convolution(self):
        start = 2.0 * pq.s
        stop = 5.0 * pq.s
        sampling_rate = 2.0 * pq.Hz
        kernel = sigproc.RectangularKernel(0.6 * pq.s)
        st = create_empty_spike_train(start, stop)

        expected_length = (stop - start) * sampling_rate - 2
        expected_bins = sp.array([2.5, 3.0, 3.5, 4.0, 4.5]) * pq.s

        binned, bins = sigproc.st_convolve(
            st, kernel, mode='valid', sampling_rate=sampling_rate)
        self.assertEqual(binned.size, expected_length)
        assert_array_almost_equal(expected_bins, bins)


if __name__ == '__main__':
    ut.main()



import hashlib
import json
import os
import tables
import time

import gui_data


class HashEntry(tables.IsDescription):
    hash = tables.StringCol(32)
    filename = tables.StringCol(992)  # 1024-32 -> long filenames are possible


class AnalysisPlugin(gui_data.DataSet):
    """ Base class for Analysis plugins. Inherit this class to create a
    plugin.

    The two most important methods are :func:`get_name` and :func:`start`.
    Both should be overridden by every plugin. The class also has
    functionality for GUI configuration and saving/restoring analysis
    results.

    The GUI configuration uses :mod:`guidata`. Because `AnalysisPlugin`
    inherits from `DataSet`, configuration options can easily be added
    directly to the class definition. For example, the following code
    creates an analysis that has two configuration options which are
    used in the start() method to print to the console::

        from spykeutils.plugin import analysis_plugin, gui_data

        class SamplePlugin(analysis_plugin.AnalysisPlugin):
            some_time = gui_data.FloatItem('Some time', default=2.0, unit='ms')
            print_more = gui_data.BoolItem('Print additional info', default=True)

            def start(self, current, selections):
                print 'The selected time is', self.some_time, 'milliseconds.'
                if self.print_more:
                    print 'This is important additional information!'


    The class attribute ``data_dir`` contains a base directory for saving
    and loading data. It is set by Spyke Viewer to the directory specified
    in the settings. When using an AnalysisPlugin without Spyke Viewer,
    the default value is an empty string (so the current directory will
    be used) and the attribute can be set to an arbitrary directory.
    """

    data_dir = ''

    def __init__(self):
        super(AnalysisPlugin, self).__init__()

    def get_name(self):
        """ Return the name of an analysis. Override to specify analysis
        name.

        :returns: The name of the plugin.
        :rtype: str
        """
        return 'Prototype Plugin'

    def get_title(self):  # Override guidata.DataSet.get_title()
        return self.get_name()

    def get_comment(self):  # Override guidata.DataSet.get_comment()
        ret = None
        if self.__doc__:
            string = self.__doc__
            if not isinstance(string, basestring):
                string = unicode(string)
            if not isinstance(string, unicode):
                string = unicode(string, 'utf-8')

            doc_lines = string.splitlines()
            # Remove empty lines at the begining of comment
            while doc_lines and not doc_lines[0].strip():
                del doc_lines[0]
            if doc_lines:
                ret = "\n".join([x.strip() for x in doc_lines])
        return ret

    def start(self, current, selections):
        """ Entry point for processing. Override with analysis code.

        :param current: This data provider is used if the analysis
            should be performed on the data currently selected in the GUI.
        :type current:
            :class:`spykeviewer.plugin_framework.data_provider.DataProvider`
        :param list selections: This parameter contains all saved
            selections. It is used if an analysis needs multiple data sets.
        """
        pass

    def configure(self):
        """ Configure the analysis. Override if a different or additional
        configuration apart from guidata is needed.
        """
        if self._items:
            return self.edit()

    def get_parameters(self):
        """ Return a dictionary of the configuration that can
        be read with :func:`deserialize_parameters`. Override both if
        non-guidata attributes need to be serialized or if some guidata
        parameters should not be serialized (e.g. they only affect the
        visual presentation).

        :returns: A dictionary of all configuration parameters.
        :rtype: dict
        """
        if not hasattr(self, '_items'):
            return {}

        ret = {}
        for i in self._items:
            v = i.get_value(self)
            if isinstance(v, str):
                ret[i._name] = unicode(v)
            else:
                ret[i._name] = v
        return ret

    def set_parameters(self, parameters):
        """ Load configuration from a dictionary that has been
        created by :func:`serialize_parameters`. Parameters that are not
        part of the guidata attributes of the plugin are ignored. Override if
        non-guidata attributes need to be serialized.

        :param dict parameters: A dictionary of all configuration
            parameters.
        """
        for n, v in parameters.iteritems():
            if hasattr(self, '_' + n):
                setattr(self, '_' + n, v)

    def _get_hash(self, selections, params, use_guiparams):
        """ Return hash and the three strings used for it
        (guidata,selections,params)
        """
        if use_guiparams:
            guidata_string = repr(sorted(self.get_parameters().items()))
        else:
            guidata_string = ''
        selection_string = json.dumps([s.data_dict() for s in selections])

        if params:
            param_string = repr(sorted(params.items()))
        else:
            param_string = ''

        md5 = hashlib.md5()
        hash_string = guidata_string + selection_string + param_string
        md5.update(hash_string)

        return md5.hexdigest(), guidata_string, selection_string, param_string

    def save(self, name, selections, params=None, save_guiparams=True):
        """ Return a HDF5 file object with parameters already stored.
        Save analysis results to this file.

        :param str name: The name of the results to save. A folder with
            this name will be used (and created if necessary) to store
            the analysis result files.
        :param sequence selections: A list of :class:`DataProvider` objects
            that are relevant for the analysis results.
        :param dict params: A dictionary, indexed by strings (which should
            be valid as python identifiers), with parameters apart from GUI
            configuration used to obtain the results. All keys have to be
            integers, floats, strings or lists of these types.
        :param bool save_guiparams: Determines if the guidata parameters of
            the class should be saved in the file.
        :returns: An open PyTables file object ready to be used to store
            data. Afterwards, the file has to be closed by calling the
            :func:`tables.File.close` method.
        :rtype: :class:`tables.File`
        """
        if not selections:
            selections = []

        if not os.path.exists(os.path.join(self.data_dir, name)):
            os.makedirs(os.path.join(self.data_dir, name))

        if params is None:
            params = {}

        # Use unicode parameters
        for n, v in params.iteritems():
            if isinstance(v, str):
                params[n] = unicode(v)

        # Create parameter hash
        hash_, guidata_string, selection_string, param_string = \
            self._get_hash(selections, params, save_guiparams)

        # File name is current time stamp
        time_stamp = time.strftime("%Y%m%d-%H%M%S")
        file_name_base = os.path.join(self.data_dir, name, time_stamp)
        file_name = file_name_base

        # Make sure not to overwrite another file
        i = 2
        while os.path.exists(file_name):
            file_name = file_name_base + '_%d' % i
            i += 1
        file_name += '.h5'

        self._add_hash_lookup_entry(name, hash_, file_name)

        h5 = tables.openFile(file_name, 'w')

        # Save guidata parameters
        paramgroup = h5.createGroup('/', 'guiparams')
        if save_guiparams:
            guiparams = self.get_parameters()
            for p, v in guiparams.iteritems():
                t = type(v)
                if t == int or t == float:
                    h5.setNodeAttr(paramgroup, p, v)
                else:
                    h5.setNodeAttr(paramgroup, p, json.dumps(v))

        # Save selections the provided by plugin
        h5.setNodeAttr('/', 'selections', selection_string)

        # Save additional parameters provided by plugin
        paramgroup = h5.createGroup('/', 'userparams')
        for p, v in params.iteritems():
            t = type(v)
            if t == int or t == float:
                h5.setNodeAttr(paramgroup, p, v)
            else:
                h5.setNodeAttr(paramgroup, p, json.dumps(v))

        # Save hash and current time
        h5.setNodeAttr('/', '_hash', hash_)
        h5.setNodeAttr('/', 'time', time.time())

        return h5

    def load(self, name, selections, params=None, consider_guiparams=True):
        """ Return the most recent HDF5 file for a certain parameter
        configuration. If no such file exists, return None. This
        function works with the files created by :func:`save`.

        :param str name: The name of the results to load.
        :param sequence selections: A list of :class:`DataProvider` objects
            that are relevant for the analysis results.
        :param dict params: A dictionary, indexed by strings (which should
            be valid as python identifiers), with parameters apart from GUI
            configuration used to obtain the results. All keys have to be
            integers, floats, strings or lists of these types.
        :param bool consider_guiparams: Determines if the guidata parameters
            of the class should be considered if they exist in the HDF5
            file. This should be set to False if :func:`save` is used with
            ``save_guiparams`` set to ``False``.
        :returns: An open PyTables file object ready to be used to read
            data. Afterwards, the file has to be closed by calling the
            :func:`tables.File.close` method. If no appropriate file
            exists, None is returned.
        :rtype: :class:`tables.File`
        """
        if not selections:
            selections = []

        if not os.path.exists(os.path.join(self.data_dir, name)):
            return None

        if params is None:
            params = {}

        # Use unicode parameters
        for n, v in params.iteritems():
            if isinstance(v, str):
                params[n] = unicode(v)

        hash_, guidata_string, selection_string, param_string =\
            self._get_hash(selections, params, consider_guiparams)

        # Loop through files and find the most recent match
        file_names = self._get_hash_file_names(name, hash_)
        newest = 0.0
        best = None
        for fn in file_names:
            with tables.openFile(fn, 'r') as h5:
                file_hash = h5.getNodeAttr('/', '_hash')

                if hash_ != file_hash:
                    continue

                # Hash is correct, check guidata parameters
                gui_params = {}
                for pname in h5.root.guiparams._v_attrs._f_list('user'):
                    v = h5.getNodeAttr('/guiparams', pname)
                    if isinstance(v, str):
                        gui_params[pname] = json.loads(v)
                    else:
                        gui_params[pname] = v

                if gui_params:
                    gui_param_string = repr(sorted(gui_params.items()))
                else:
                    gui_param_string = ''

                if gui_param_string != guidata_string:
                    continue

                # Check selections
                file_selections = h5.getNodeAttr('/', 'selections')
                if file_selections != selection_string:
                    continue

                # Check custom parameters
                file_params = {}
                for pname in h5.root.userparams._v_attrs._f_list('user'):
                    v = h5.getNodeAttr('/userparams', pname)
                    if isinstance(v, str):
                        file_params[pname] = json.loads(v)
                    else:
                        file_params[pname] = v

                if file_params:
                    file_param_string = repr(sorted(file_params.items()))
                else:
                    file_param_string = ''

                if file_param_string != param_string:
                    continue

                # Make sure the most recent file is used
                analysis_time = h5.getNodeAttr('/', 'time')
                if analysis_time < newest:
                    continue

                best = fn
                newest = analysis_time

        if best:
            return tables.openFile(best, 'r')
        return None

    @classmethod
    def _create_hash_lookup_file(cls, name):
        """ (Re)creates a hash lookup file for a results directory. This
        file contains all file hashes in the directory so that the
        correct file for a given parameter set can be found quickly.

        :param str name: The name of the results.
        """
        name = os.path.join(cls.data_dir, name)
        hashfile_name = os.path.join(name, 'hash.h5')
        hash_file = tables.openFile(hashfile_name, mode='w')
        table = hash_file.createTable('/', 'lookup_table', HashEntry,
                                      title='Hash lookup')

        # Loop through files and write hashes
        file_names = [os.path.join(name, f) for f in os.listdir(name)]
        entry = table.row
        for fn in file_names:
            if not fn.endswith('.h5') or fn == 'hash.h5':
                continue

            try:
                with tables.openFile(fn, 'r') as h5:
                    file_hash = h5.getNodeAttr('/', '_hash')
                    entry['hash'] = file_hash
                    entry['filename'] = fn
                    entry.append()
            except:
                pass  # Not a valid data file, no problem

        hash_file.close()

    @classmethod
    def _add_hash_lookup_entry(cls, name, hash_, file_name):
        """ Add a new entry to the hash lookup file.

        :param str name: The name of the results.
        :param str hash_: The hash of the parameters.
        :param str file_name: The file name of the results.
        """
        hashfile_name = os.path.join(cls.data_dir, name, 'hash.h5')
        if not os.path.exists(hashfile_name):
            cls._create_hash_lookup_file(name)

        hash_file = tables.openFile(hashfile_name, mode='r+')
        table = hash_file.root.lookup_table

        # Add entry
        entry = table.row
        entry['hash'] = hash_
        entry['filename'] = file_name
        entry.append()

        hash_file.close()

    @classmethod
    def _get_hash_file_names(cls, name, hash_, _recurse=False):
        """ Return a list of file names for a parameter hash. If no hash
        lookup file exists, it will be created. If it can not be
        created, a list HDF5 files in the directory will be returned.

        :param str name: The name of the results.
        :param str hash_: The hash of the parameters.
        :param bool _recurse: Internal guard against infinite recursion.
        """
        dataname = name
        name = os.path.join(cls.data_dir, name)
        hashfile_name = os.path.join(name, 'hash.h5')
        if not os.path.exists(hashfile_name):
            try:
                cls._create_hash_lookup_file(name)
            except:
                return [os.path.join(name, f) for f in os.listdir(name)
                        if f.endswith('.h5') and not f == 'hash.h5']

        hash_file = tables.openFile(hashfile_name, mode='r')
        table = hash_file.root.lookup_table

        files = [row['filename'] for row in
                 table.where('hash == "%s"' % hash_)]

        ret = []
        for f in files:
            if os.path.exists(f):
                ret.append(f)
            elif not _recurse:
                hash_file.close()
                try:
                    cls._create_hash_lookup_file(name)
                except:
                    return [os.path.join(name, f) for f in os.listdir(name)
                            if f.endswith('.h5') and not f == 'hash.h5']
                return cls._get_hash_file_names(dataname, hash_, True)

        hash_file.close()
        return ret


import scipy as sp

from guiqwt.builder import make
from guiqwt.baseplot import BasePlot
from guiqwt.plot import BaseCurveWidget

import quantities as pq

from .. import SpykeException
from dialog import PlotDialog
import helper


@helper.needs_qt
def isi(trains, bin_size, cut_off, bar_plot=False, time_unit=pq.ms):
    """ Create a plot dialog with an interspike interval histogram.

    :param dict trains: Dictionary with lists of spike trains indexed by
        units for which to display ISI histograms
    :param bin_size: Bin size (time)
    :type bin_size: Quantity scalar
    :param cut_off: End of histogram (time)
    :type bin_size: Quantity scalar
    :param bool bar_plot: If ``True``, create a bar ISI histogram for each
        index in ``trains``. Else, create a line ISI histogram.
    :param Quantity time_unit: Unit of X-Axis.
    """
    if not trains:
        raise SpykeException('No spike trains for ISI histogram')

    win_title = 'ISI Histogram | Bin size: ' + str(bin_size)
    win = PlotDialog(toolbar=True, wintitle=win_title, min_plot_width=150,
                     min_plot_height=100)
    bin_size = bin_size.rescale(time_unit)
    cut_off = cut_off.rescale(time_unit)
    bins = sp.arange(0 * time_unit, cut_off, bin_size) * time_unit

    legends = []
    if bar_plot:
        ind = 0
        columns = int(sp.sqrt(len(trains)))
        for i, train_list in trains.iteritems():
            pW = BaseCurveWidget(win)
            plot = pW.plot
            intervals = []
            for t in train_list:
                t = t.rescale(time_unit)
                sTrain = sp.asarray(t)
                sTrain.sort()
                intervals.extend(sp.diff(sTrain))

            (isi, bins) = sp.histogram(intervals, bins)

            if i and hasattr(i, 'name') and i.name:
                name = i.name
            else:
                name = 'Unknown'

            show_isi = list(isi)
            show_isi.insert(0, show_isi[0])
            curve = make.curve(
                bins, show_isi, name, color='k',
                curvestyle="Steps", shade=1.0)
            plot.add_item(curve)

            # Create legend
            color = helper.get_object_color(i)
            color_curve = make.curve(
                [], [], name, color, 'NoPen', linewidth=1, marker='Rect',
                markerfacecolor=color, markeredgecolor=color)
            plot.add_item(color_curve)
            legends.append(make.legend(restrict_items=[color_curve]))
            plot.add_item(legends[-1])

            # Prepare plot
            plot.set_antialiasing(False)
            scale = plot.axisScaleDiv(BasePlot.Y_LEFT)
            plot.setAxisScale(BasePlot.Y_LEFT, 0, scale.upperBound())
            if ind % columns == 0:
                plot.set_axis_title(BasePlot.Y_LEFT, 'Number of intervals')
            if ind >= len(trains) - columns:
                plot.set_axis_title(BasePlot.X_BOTTOM, 'Interval length')
                plot.set_axis_unit(
                    BasePlot.X_BOTTOM, time_unit.dimensionality.string)

            win.add_plot_widget(pW, ind, column=ind % columns)
            ind += 1
    else:
        pW = BaseCurveWidget(win)
        plot = pW.plot
        legend_items = []

        for i, train_list in trains.iteritems():
            intervals = []
            for t in train_list:
                t = t.rescale(time_unit)
                sTrain = sp.asarray(t)
                sTrain.sort()
                intervals.extend(sp.diff(sTrain))

            (isi, bins) = sp.histogram(intervals, bins)

            if i and hasattr(i, 'name') and i.name:
                name = i.name
            else:
                name = 'Unknown'
            color = helper.get_object_color(i)

            curve = make.curve(bins, isi, name, color=color)
            legend_items.append(curve)
            plot.add_item(curve)

        win.add_plot_widget(pW, 0)

        legends.append(make.legend(restrict_items=legend_items))
        plot.add_item(legends[-1])

        plot.set_antialiasing(True)
        plot.set_axis_title(BasePlot.Y_LEFT, 'Number of intervals')
        plot.set_axis_title(BasePlot.X_BOTTOM, 'Interval length')
        plot.set_axis_unit(BasePlot.X_BOTTOM, time_unit.dimensionality.string)

    win.add_custom_curve_tools()
    win.add_legend_option(legends, True)
    win.show()

    if bar_plot and len(trains) > 1:
        win.add_x_synchronization_option(True, range(len(trains)))
        win.add_y_synchronization_option(False, range(len(trains)))

    return win


"""
This module contains some utility functions that are usefult in plot creation.
"""

import functools

import scipy as sp

from PyQt4.QtGui import QApplication
from PyQt4 import QtCore
from PyQt4.QtGui import QProgressDialog
from guiqwt.builder import make

from ..progress_indicator import (ProgressIndicator,
                                  CancelException)


class _MarkerName:
    """ Helper class to create marker name functions for different strings.
    """

    def __init__(self, name):
        self.name = name

    #noinspection PyUnusedLocal
    def get_name(self, x, y):
        return self.name


def _needs_qt(function):
    @functools.wraps(function)
    def inner(*args, **kwargs):
        if not QApplication.instance():
            _needs_qt.app = QApplication([])
        ret = function(*args, **kwargs)
        if _needs_qt.app:
            _needs_qt.app.exec_()
        return ret

    return inner


_needs_qt.app = None

# Make need_qt decorator preserve signature if decorator package is available
try:
    from decorator import FunctionMaker

    def decorator_apply(dec, func):
        return FunctionMaker.create(
            func, 'return decorated(%(signature)s)',
            dict(decorated=dec(func)), __wrapped__=func)

    def needs_qt(func):
        """ Decorator for functions making sure that an initialized PyQt exists.
        """
        return decorator_apply(_needs_qt, func)
except ImportError:
    needs_qt = _needs_qt


# Optimum contrast color palette (without white and black), see
# http://web.media.mit.edu/~wad/color/palette.html
__default_colors = [
    '#575757',  # Dark Gray
    '#ad2323',  # Red
    '#2a4bd7',  # Blue
    '#296914',  # Green
    '#614a19',  # Brown (lower R to better distinguish from purple)
    '#8126c0',  # Purple
    '#a0a0a0',  # Light Gray
    '#81c57a',  # Light Green
    '#9dafff',  # Light Blue
    '#29d0d0',  # Cyan
    '#ff9233',  # Orange
    '#ffee33',  # Yellow
    '#b6ab88',  # Tan (darkened for better visibility on white background)
    '#ff89d1']  # Pink (darkened for better visibility on white background)

__colors = __default_colors


def get_color(entity_id):
    """ Return a color for an int.
    """
    return __colors[entity_id % len(__colors)]


def get_object_color(unit):
    """ Return a color for a Neo object, based on the 'unique_id'
    annotation. If the annotation is not present, return a color based
    on the hash of the object.
    """
    try:
        if 'unique_id' in unit.annotations:
            return get_color(unit.annotations['unique_id'])
    except Exception:
        return get_color(hash(unit))
    return get_color(hash(unit))


def set_color_scheme(colors):
    """ Set the color scheme used in plots.

    :param sequence colors: A list of strings with HTML-style color codes
        (e.g. ``'#ffffff'`` for white). If this is ``None`` or empty,
        the default color scheme will be set.
    """
    global __colors
    global __default_colors
    if not colors:
        __colors = __default_colors
    else:
        __colors = colors


def add_events(plot, events, units=None):
    """ Add Event markers to a guiqwt plot.

    :param plot: The plot object.
    :type plot: :class:`guiqwt.baseplot.BasePlot`
    :param sequence events: The events (neo :class:`neo.Event` objects).
    :param Quantity units: The x-scale of the plot. If this is ``None``,
        the time unit of the events will be use.
    """
    for m in events:
        nameObject = _MarkerName(m.label)
        if units:
            time = m.time.rescale(units)
        else:
            time = m.time
        plot.add_item(
            make.marker(
                (time, 0), nameObject.get_name,
                movable=False, markerstyle='|', color='k', linestyle=':',
                linewidth=1))


def add_spikes(plot, train, color='k', spike_width=1, spike_height=20000,
               y_offset=0, name='', units=None):
    """ Add all spikes from a spike train to a guiqwt plot as vertical lines.

    :param plot: The plot object.
    :type plot: :class:`guiqwt.baseplot.BasePlot`
    :param train: A spike train with the spike times to show.
    :type train: :class:`neo.core.SpikeTrain`
    :param str color: The color for the spikes.
    :param int spike_width: The width of the shown spikes in pixels.
    :param int spike_height: The height of the shown spikes in pixels.
    :param float y_offset: An offset for the drawing position on the y-axis.
    :param str name: The name of the spike train.
    :param Quantity units: The x-scale of the plot. If this is ``None``,
        the time unit of the events will be use.
    :returns: The plot item added for the spike train
    """
    if units:
        train = train.rescale(units)

    spikes = make.curve(
        train, sp.zeros(len(train)) + y_offset,
        name, 'k', 'NoPen', linewidth=0, marker='Rect',
        markerfacecolor=color, markeredgecolor=color)

    s = spikes.symbol()
    s.setSize(spike_width - 1, spike_height)
    spikes.setSymbol(s)
    plot.add_item(spikes)

    return spikes


def add_epochs(plot, epochs, units=None):
    """ Add Epoch markers to a guiqwt plot.

    :param plot: The plot object.
    :type plot: :class:`guiqwt.baseplot.BasePlot`
    :param sequence epochs: The epochs (neo :class:`neo.Epoch` objects).
    :param units: The x-scale of the plot. If this is ``None``,
        the time unit of the events will be use.
    """
    for e in epochs:
        if units:
            start = e.time.rescale(units)
            end = (e.time + e.duration).rescale(units)
        else:
            start = e.time
            end = e.time + e.duration
        time = (start + end) / 2.0

        o = make.range(start, end)
        o.setTitle(e.label)
        o.set_readonly(True)
        o.set_movable(False)
        o.set_resizable(False)
        o.set_selectable(False)
        o.set_rotatable(False)
        plot.add_item(o)

        nameObject = _MarkerName(e.label)
        plot.add_item(make.marker(
            (time, 0), nameObject.get_name,
            movable=False, markerstyle='|', color='k', linestyle='NoPen',
            linewidth=1))


def make_window_legend(win, objects, show_option=None):
    """ Create a legend in a PlotDialog for a given sequence of neo objects.

    :param win: The window where the legend will be added.
    :type win: :class:`spykeutils.plot.dialogs.PlotDialog`
    :param sequence objects: A list of neo objects which will be included in
        the legend.
    :param bool show_option: Determines whether a toggle for the legend
        will be shown (if the parameter is not ``None``) and if the legend
        is visible initially (``True``/``False``).
    """
    if not objects:
        return

    legend = []
    for u in objects:
        if u is not None:
            name = u.name
        else:
            name = 'No identifier'
        legend.append((get_object_color(u), name))
    win.add_color_legend(legend, show_option)


class ProgressIndicatorDialog(ProgressIndicator, QProgressDialog):
    """ This class implements
    :class:`spykeutils.progress_indicator.ProgressIndicator` as a
    ``QProgressDialog``. It can be used to indicate progress in a graphical
    user interface. Qt needs to be initialized in order to use it.
    """
    def __init__(self, parent, title='Processing...'):
        QProgressDialog.__init__(self, parent)
        self.setWindowTitle(title)
        self.setMinimumWidth(500)
        self.setAutoReset(False)

    def set_ticks(self, ticks):
        self.setMaximum(ticks)
        if self.isVisible():
            self.setValue(0)

    def begin(self, title='Processing...'):
        self.setWindowTitle(title)
        self.setLabelText('')
        self.setValue(0)

        if not self.isVisible():
            self.reset()
            self.open()

        QtCore.QCoreApplication.instance().processEvents()

    def step(self, num_steps=1):
        if not self.isVisible():
            return

        self.setValue(self.value() + num_steps)
        QtCore.QCoreApplication.instance().processEvents()
        if self.wasCanceled():
            self.done()
            raise CancelException()

        super(ProgressIndicatorDialog, self).step()

    def set_status(self, status):
        self.setLabelText(status)
        QtCore.QCoreApplication.instance().processEvents()

    def done(self):
        self.reset()



from __future__ import division

import scipy as sp
import quantities as pq

from guiqwt.builder import make
from guiqwt.baseplot import BasePlot
from guiqwt.plot import BaseCurveWidget

from ..progress_indicator import ProgressIndicator
from .. import conversions
from .. import SpykeException
from dialog import PlotDialog
import helper


@helper.needs_qt
def signals(signals, events=None, epochs=None, spike_trains=None,
            spikes=None, show_waveforms=True, use_subplots=True,
            subplot_names=True, time_unit=pq.s, y_unit=None, progress=None):
    """ Create a plot from a list of analog signals.

    :param list signals: The list of :class:`neo.core.AnalogSignal` objects
        to plot.
    :param sequence events: A list of Event objects to be included in the
        plot.
    :param sequence epochs: A list of Epoch objects to be included in the
        plot.
    :param list spike_trains: A list of :class:`neo.core.SpikeTrain` objects
        to be included in the plot. The ``unit`` property (if it exists) is
        used for color and legend entries.
    :param list spikes: A list :class:`neo.core.Spike` objects to be included
        in the plot. The ``unit`` property (if it exists) is used for color
        and legend entries.
    :param bool show_waveforms: Determines if spikes from
        :class:`neo.core.Spike` and :class:`neo.core.SpikeTrain` objects are
        shown as waveforms (if available) or vertical lines.
    :param bool use_subplots: Determines if a separate subplot for is created
        each signal.
    :param bool subplot_names: Only valid if ``use_subplots`` is True.
        Determines if signal (or channel) names are shown for subplots.
    :param Quantity time_unit: The unit of the x axis.
    :param progress: Set this parameter to report progress.
    :type progress: :class:`spykeutils.progress_indicator.ProgressIndicator`
    """
    if not signals:
        raise SpykeException(
            'Cannot create signal plot: No signal data provided!')
    if not progress:
        progress = ProgressIndicator()

    # Plot title
    win_title = 'Analog Signal'
    if len(set((s.recordingchannel for s in signals))) == 1:
        if signals[0].recordingchannel and signals[0].recordingchannel.name:
            win_title += ' | Recording Channel: %s' %\
                         signals[0].recordingchannel.name
    if len(set((s.segment for s in signals))) == 1:
        if signals[0].segment and signals[0].segment.name:
            win_title += ' | Segment: %s' % signals[0].segment.name
    win = PlotDialog(toolbar=True, wintitle=win_title)

    if events is None:
        events = []
    if epochs is None:
        epochs = []
    if spike_trains is None:
        spike_trains = []
    if spikes is None:
        spikes = []

    if show_waveforms:
        for st in spike_trains:
            if st.waveforms is not None:
                spikes.extend(conversions.spike_train_to_spikes(st))
        spike_trains = []
    else:
        unit_spikes = {}
        for s in spikes:
            unit_spikes.setdefault(s.unit, []).append(s)
        for sps in unit_spikes.itervalues():
            spike_trains.append(conversions.spikes_to_spike_train(sps, False))
        spikes = []

    channels = range(len(signals))

    channel_indices = []
    for s in signals:
        if not s.recordingchannel:
            channel_indices.append(-1)
        else:
            channel_indices.append(s.recordingchannel.index)

    # Heuristic: If multiple channels have the same index, use channel order
    # as index for spike waveforms
    nonindices = max(0, channel_indices.count(-1) - 1)
    if len(set(channel_indices)) != len(channel_indices) - nonindices:
        channel_indices = range(len(signals))

    progress.set_ticks((len(spike_trains) + len(spikes) + 1) * len(channels))

    offset = 0 * signals[0].units
    if use_subplots:
        plot = None
        for c in channels:
            pW = BaseCurveWidget(win)
            plot = pW.plot

            if subplot_names:
                if signals[c].name:
                    win.set_plot_title(plot, signals[c].name)
                elif signals[c].recordingchannel:
                    if signals[c].recordingchannel.name:
                        win.set_plot_title(
                            plot, signals[c].recordingchannel.name)

            sample = (1 / signals[c].sampling_rate).simplified
            x = (sp.arange(signals[c].shape[0])) * sample + signals[c].t_start
            x.units = time_unit

            helper.add_epochs(plot, epochs, x.units)
            if y_unit is not None:
                plot.add_item(make.curve(x, signals[c].rescale(y_unit)))
            else:
                plot.add_item(make.curve(x, signals[c]))
            helper.add_events(plot, events, x.units)

            _add_spike_waveforms(
                plot, spikes, x.units, channel_indices[c], offset, progress)

            for train in spike_trains:
                color = helper.get_object_color(train.unit)
                helper.add_spikes(plot, train, color, units=x.units)
                progress.step()

            win.add_plot_widget(pW, c)
            plot.set_axis_unit(
                BasePlot.Y_LEFT, signals[c].dimensionality.string)
            progress.step()

        plot.set_axis_title(BasePlot.X_BOTTOM, 'Time')
        plot.set_axis_unit(BasePlot.X_BOTTOM, x.dimensionality.string)
    else:
        channels.reverse()

        pW = BaseCurveWidget(win)
        plot = pW.plot

        helper.add_epochs(plot, epochs, time_unit)

        # Find plot y offset
        max_offset = 0 * signals[0].units
        for i, c in enumerate(channels[1:], 1):
            cur_offset = signals[channels[i - 1]].max() - signals[c].min()
            if cur_offset > max_offset:
                max_offset = cur_offset

        offset -= signals[channels[0]].min()

        for c in channels:
            sample = (1 / signals[c].sampling_rate).simplified
            x = (sp.arange(signals[c].shape[0])) * sample + signals[c].t_start
            x.units = time_unit

            if y_unit is not None:
                plot.add_item(
                    make.curve(x, (signals[c] + offset).rescale(y_unit)))
            else:
                plot.add_item(make.curve(x, signals[c] + offset))
            _add_spike_waveforms(
                plot, spikes, x.units, channel_indices[c], offset, progress)
            offset += max_offset
            progress.step()

        helper.add_events(plot, events, x.units)

        for train in spike_trains:
            color = helper.get_object_color(train.unit)
            helper.add_spikes(plot, train, color, units=x.units)
            progress.step()

        win.add_plot_widget(pW, 0)

        plot.set_axis_title(BasePlot.X_BOTTOM, 'Time')
        plot.set_axis_unit(BasePlot.X_BOTTOM, x.dimensionality.string)
        plot.set_axis_unit(BasePlot.Y_LEFT, signals[0].dimensionality.string)

    win.add_custom_curve_tools()

    units = set([s.unit for s in spike_trains])
    units = units.union([s.unit for s in spikes])

    progress.done()

    helper.make_window_legend(win, units, False)
    win.show()

    if use_subplots:
        win.add_x_synchronization_option(True, channels)
        win.add_y_synchronization_option(False, channels)

    return win


def _add_spike_waveforms(plot, spikes, x_units, channel, offset, progress):
    for spike in spikes:
        if spike.waveform is None or not spike.sampling_rate:
            continue
        if channel < 0 or spike.waveform.shape[1] <= channel:
            continue

        color = helper.get_object_color(spike.unit)
        if spike.left_sweep:
            lsweep = spike.left_sweep
        else:
            lsweep = 0.0 * pq.ms
        start = (spike.time - lsweep).rescale(x_units)
        stop = (spike.waveform.shape[0] / spike.sampling_rate +
                spike.time - lsweep).rescale(x_units)
        spike_x = sp.arange(
            start, stop,
            (1.0 / spike.sampling_rate).rescale(x_units)) * x_units

        plot.add_item(
            make.curve(spike_x, spike.waveform[:, channel] + offset,
                       color=color, linewidth=2))
        progress.step()